@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 include ('../inc/includes.php');
 
 header("Content-Type: text/html; charset=UTF-8");
@@ -43,8 +39,6 @@ Session::checkCentralAccess();
 
 // Make a select box
 if ($_POST["idtable"] && class_exists($_POST["idtable"])) {
-   $table = getTableForItemType($_POST["idtable"]);
-
    // Link to user for search only > normal users
    $link = "getDropdownValue.php";
 
@@ -53,6 +47,9 @@ if ($_POST["idtable"] && class_exists($_POST["idtable"])) {
    }
 
    $rand     = mt_rand();
+   if (isset($_POST['rand'])) {
+      $rand = $_POST['rand'];
+   }
 
    $field_id = Html::cleanId("dropdown_".$_POST["name"].$rand);
 
@@ -70,6 +67,9 @@ if ($_POST["idtable"] && class_exists($_POST["idtable"])) {
    if (isset($_POST['condition'])) {
       $p['condition'] = $_POST['condition'];
    }
+   if (isset($_POST['used'])) {
+      $_POST['used'] = Toolbox::jsonDecode($_POST['used'], true);
+   }
    if (isset($_POST['used'][$_POST['idtable']])) {
       $p['used'] = $_POST['used'][$_POST['idtable']];
    }
