@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,9 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
- * @since version 0.84
-* @brief
-*/
+/**
+ * @since 0.84
+ */
 
 $AJAX_INCLUDE = 1;
 
@@ -78,14 +77,15 @@ if (class_exists($_POST["itemtype"])
              WHERE `glpi_networkports_networkports`.`id` IS NULL
                    AND `glpi_networkports`.`id` IS NOT NULL
                    AND `glpi_networkports`.`id` <> '".$_POST['networkports_id']."'
-                   AND `$table`.`is_deleted` = '0'
-                   AND `$table`.`is_template` = '0'
+                   AND `$table`.`is_deleted` = 0
+                   AND `$table`.`is_template` = 0
              ORDER BY `glpi_networkports`.`id`";
 
    $result = $DB->query($query);
 
    echo "<br>";
 
+   $values = [];
    if ($DB->numrows($result)) {
       while ($data = $DB->fetch_assoc($result)) {
          // Device name + port name
