@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 use Glpi\Event;
 
 include ('../inc/includes.php');
@@ -49,8 +45,6 @@ if (!isset($_GET["withtemplate"])) {
 }
 
 $contract         = new Contract();
-$contractitem     = new Contract_Item();
-$contractsupplier = new Contract_Supplier();
 
 if (isset($_POST["add"])) {
    $contract->check(-1, CREATE, $_POST);
