@@ -1,77 +1,78 @@
 <?php
-/*
- * @version $Id: report.year.list.php 20130 2013-02-04 16:55:15Z moyo $
- -------------------------------------------------------------------------
- GLPI - Gestionnaire Libre de Parc Informatique
- Copyright (C) 2003-2013 by the INDEPNET Development Team.
-
- http://indepnet.net/   http://glpi-project.org
- -------------------------------------------------------------------------
-
- LICENSE
-
- This file is part of GLPI.
-
- GLPI is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
-
- GLPI is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with GLPI. If not, see <http://www.gnu.org/licenses/>.
- --------------------------------------------------------------------------
+/**
+ * ---------------------------------------------------------------------
+ * GLPI - Gestionnaire Libre de Parc Informatique
+ * Copyright (C) 2015-2018 Teclib' and contributors.
+ *
+ * http://glpi-project.org
+ *
+ * based on GLPI - Gestionnaire Libre de Parc Informatique
+ * Copyright (C) 2003-2014 by the INDEPNET Development Team.
+ *
+ * ---------------------------------------------------------------------
+ *
+ * LICENSE
+ *
+ * This file is part of GLPI.
+ *
+ * GLPI is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * GLPI is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GLPI. If not, see <http://www.gnu.org/licenses/>.
+ * ---------------------------------------------------------------------
  */
 
-// ----------------------------------------------------------------------
-// Original Author of file:
-// Purpose of file:
-// ----------------------------------------------------------------------
+include ("../inc/includes.php");
 
-define ('GLPI_ROOT', '..');
-include (GLPI_ROOT . "/inc/includes.php");
+Session::checkRight("reports", READ);
 
-Session::checkRight("reports", "r");
-
-Html::header($LANG['Menu'][6],$_SERVER['PHP_SELF'],"utils","report");
+Html::header(Report::getTypeName(Session::getPluralNumber()), $_SERVER['PHP_SELF'], "tools", "report");
 
 Report::title();
 
-$items = array('Computer', 'Monitor', 'NetworkEquipment', 'Peripheral', 'Phone', 'Printer',
-               'SoftwareLicense');
+$items = $CFG_GLPI["report_types"];
 
-# Titre
-echo "<div class='center b'><big>".$LANG['reports'][57]."</big></div><br><br>";
+// Titre
+echo "<div class='center b spaced'><big>".__('Device list')."</big></div>";
 
-# Request All
-if ((isset($_POST["item_type"][0]) && $_POST["item_type"][0] == '0')
+// Request All
+if ((isset($_POST["item_type"][0]) && ($_POST["item_type"][0] == 0))
     || !isset($_POST["item_type"])) {
    $_POST["item_type"] = $items;
 }
 
 if (isset($_POST["item_type"]) && is_array($_POST["item_type"])) {
-   $query = array();
+   $query = [];
    foreach ($_POST["item_type"] as $key => $val) {
-      if (in_array($val,$items)) {
+      if (in_array($val, $items)) {
          $itemtable = getTableForItemType($val);
 
-         $deleted_field="`$itemtable`.`is_deleted`";
-         $location_field="`glpi_locations`.`completename`";
-         $add_leftjoin="LEFT JOIN `glpi_locations`
-                            ON (`$itemtable`.`locations_id` = `glpi_locations`.`id`)";
-         $template_condition="`$itemtable`.`is_template` ='0'";
-         if ($val=='SoftwareLicense') {
-            $deleted_field="`glpi_softwares`.`is_deleted`";
-            $location_field="''";
-            $add_leftjoin="LEFT JOIN `glpi_softwares` ON (`glpi_softwares`.`id`
+         $deleted_field       = "`$itemtable`.`is_deleted`";
+         $location_field      = "''";
+         $add_leftjoin        = "";
+         $template_condition  = '1';
+         if ($val != 'Project') {
+            $location_field      = "`glpi_locations`.`completename`";
+            $template_condition  = "`$itemtable`.`is_template` = 0";
+            $add_leftjoin        = "LEFT JOIN `glpi_locations`
+                                       ON (`$itemtable`.`locations_id` = `glpi_locations`.`id`)";
+         }
+         if ($val == 'SoftwareLicense') {
+            $deleted_field       = "`glpi_softwares`.`is_deleted`";
+            $location_field      = "''";
+            $add_leftjoin        = "LEFT JOIN `glpi_softwares`
+                                       ON (`glpi_softwares`.`id`
                                                       = `glpi_softwarelicenses`.`softwares_id`)";
-            $template_condition="`glpi_softwares`.`is_template` ='0'";
+            $template_condition  = "`glpi_softwares`.`is_template` = 0";
          }
-
 
 
          $query[$val] = "SELECT `$itemtable`.`name` AS itemname,
@@ -97,12 +98,12 @@ if (isset($_POST["item_type"]) && is_array($_POST["item_type"])) {
                          LEFT JOIN `glpi_entities`
                                ON (`$itemtable`.`entities_id` = `glpi_entities`.`id`)
                          WHERE $template_condition ".
-                               getEntitiesRestrictRequest("AND",$itemtable);
+                               getEntitiesRestrictRequest("AND", $itemtable);
 
-         if (isset($_POST["annee"][0]) && $_POST["annee"][0] != 'toutes') {
+         if (isset($_POST["year"][0]) && ($_POST["year"][0] != 0)) {
             $query[$val] .= " AND ( ";
-            $first = true;
-            foreach ($_POST["annee"] as $key2 => $val2) {
+            $first        = true;
+            foreach ($_POST["year"] as $val2) {
                if (!$first) {
                   $query[$val] .= " OR ";
                } else {
@@ -124,21 +125,21 @@ if (isset($query) && count($query)) {
       $result = $DB->query($val);
       if ($result && $DB->numrows($result)) {
          $item = new $key();
-         echo "<div class='center b'>".$item->getTypeName()."</div>";
+         echo "<div class='center b'>".$item->getTypeName(1)."</div>";
          echo "<table class='tab_cadre_fixehov'>";
-         echo "<tr><th>".$LANG['common'][16]."</th>";
-         echo "<th>".$LANG['common'][28]."</th>";
+         echo "<tr><th>".__('Name')."</th>";
+         echo "<th>".__('Deleted')."</th>";
          if ($display_entity) {
-            echo "<th>".$LANG['entity'][0]."</th>";
+            echo "<th>".__('Entity')."</th>";
          }
-         echo "<th>".$LANG['common'][15]."</th>";
-         echo "<th>".$LANG['financial'][14]."</th>";
-         echo "<th>".$LANG['financial'][80]."</th>";
-         echo "<th>".$LANG['financial'][6]."</th>";
-         echo "<th>".$LANG['search'][8]."</th>";
-         echo "<th>".$LANG['search'][9]."</th></tr>";
+         echo "<th>".__('Location')."</th>";
+         echo "<th>".__('Date of purchase')."</th>";
+         echo "<th>".__('Warranty expiration date')."</th>";
+         echo "<th>".__('Contract type')."</th>";
+         echo "<th>".__('Start date')."</th>";
+         echo "<th>".__('End date')."</th></tr>";
 
-         while ($data = $DB->fetch_array($result)) {
+         while ($data = $DB->fetch_assoc($result)) {
             echo "<tr class='tab_bg_1'>";
             if ($data['itemname']) {
                echo "<td> ".$data['itemname']."</td>";
@@ -148,11 +149,7 @@ if (isset($query) && count($query)) {
             echo "<td class='center'>".Dropdown::getYesNo($data['itemdeleted'])."</td>";
 
             if ($display_entity) {
-               if ($data['entID'] == 0) {
-                  echo "<td>".$LANG['entity'][2]."</td>";
-               } else {
-                  echo "<td>".$data['entname']."</td>";
-               }
+               echo "<td>".$data['entname']."</td>";
             }
 
             if ($data['location']) {
@@ -202,4 +199,3 @@ if (isset($query) && count($query)) {
 }
 
 Html::footer();
-?>
\ No newline at end of file
