@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 if (!defined('GLPI_ROOT')) {
    die("Sorry. You can't access this file directly");
 }
@@ -78,7 +74,7 @@ class Alert extends CommonDBTM {
    /**
     * Clear all alerts  for an item
     *
-    * @since version 0.84
+    * @since 0.84
     *
     * @param string  $itemtype ID of the type to clear
     * @param integer $ID       ID of the item to clear
@@ -92,11 +88,12 @@ class Alert extends CommonDBTM {
 
    static function dropdown($options = []) {
 
-      $p = [];
-      $p['name']           = 'alert';
-      $p['value']          = 0;
-      $p['display']        = true;
-      $p['inherit_parent'] = false;
+      $p = [
+         'name'           => 'alert',
+         'value'          => 0,
+         'display'        => true,
+         'inherit_parent' => false,
+      ];
 
       if (count($options)) {
          foreach ($options as $key => $val) {
@@ -104,6 +101,8 @@ class Alert extends CommonDBTM {
          }
       }
 
+      $times = [];
+
       if ($p['inherit_parent']) {
          $times[Entity::CONFIG_PARENT] = __('Inheritance of the parent entity');
       }
@@ -126,10 +125,12 @@ class Alert extends CommonDBTM {
     */
    static function dropdownYesNo($options = []) {
 
-      $p['name']           = 'alert';
-      $p['value']          = 0;
-      $p['display']        = true;
-      $p['inherit_parent'] = false;
+      $p = [
+         'name'           => 'alert',
+         'value'          => 0,
+         'display'        => true,
+         'inherit_parent' => false,
+      ];
 
       if (count($options)) {
          foreach ($options as $key => $val) {
@@ -137,6 +138,8 @@ class Alert extends CommonDBTM {
          }
       }
 
+      $times = [];
+
       if ($p['inherit_parent']) {
          $times[Entity::CONFIG_PARENT] = __('Inheritance of the parent entity');
       }
@@ -159,11 +162,13 @@ class Alert extends CommonDBTM {
     */
    static function dropdownIntegerNever($name, $value, $options = []) {
 
-      $p['min']      = 1;
-      $p['max']      = 100;
-      $p['step']     = 1;
-      $p['toadd']    = [];
-      $p['display']  = true;
+      $p = [
+         'min'     => 1,
+         'max'     => 100,
+         'step'    => 1,
+         'toadd'   => [],
+         'display' => true,
+      ];
 
       if (isset($options['inherit_parent']) && $options['inherit_parent']) {
          $p['toadd'][-2] = __('Inheritance of the parent entity');
@@ -211,7 +216,7 @@ class Alert extends CommonDBTM {
    /**
     * Get date of alert
     *
-    * @since version 0.84
+    * @since 0.84
     *
     * @param string  $itemtype (default '')
     * @param integer $items_id (default '')
