@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -31,7 +31,7 @@
  */
 
 /**
- * @since version 9.1
+ * @since 9.1
  */
 
 class APIRest extends API {
@@ -55,6 +55,23 @@ class APIRest extends API {
       return __('Rest API');
    }
 
+   /**
+    * Upload and validate files from request and append to $this->parameters['input']
+    *
+    * @return void
+    */
+   public function manageUploadedFiles() {
+      foreach (array_keys($_FILES) as $filename) {
+         $upload_result
+            = GLPIUploadHandler::uploadFiles(['name'           => $filename,
+                                              'print_response' => false]);
+         foreach ($upload_result as $uresult) {
+            $this->parameters['input']->_filename[] = $uresult[0]->name;
+            $this->parameters['input']->_prefix_filename[] = $uresult[0]->prefix;
+         }
+         $this->parameters['upload_result'][] = $upload_result;
+      }
+   }
 
    /**
     * Parse url and http body to retrieve :
@@ -99,6 +116,8 @@ class APIRest extends API {
 
       // retrieve session (if exist)
       $this->retrieveSession();
+      $this->initApi();
+      $this->manageUploadedFiles();
 
       // retrieve param who permit session writing
       if (isset($this->parameters['session_write'])) {
@@ -190,8 +209,8 @@ class APIRest extends API {
          return $this->returnResponse($response, $code, $additionalheaders);
 
       } else if ($resource === "lostPassword") {
-         if ($this->verb != 'PUT') {
-            // forbid password reset when HTTP verb is not PUT
+         if ($this->verb != 'PUT' && $this->verb != 'PATCH') {
+            // forbid password reset when HTTP verb is not PUT or PATCH
             return $this->returnError(__("Only HTTP verb PUT is allowed"));
          }
          return $this->returnResponse($this->lostPassword($this->parameters));
@@ -205,8 +224,8 @@ class APIRest extends API {
          switch ($this->verb) {
             default:
             case "GET" : // retrieve item(s)
-               if (($id > 0)
-                   || (($id == 0) && ($itemtype == "Entity"))) {
+               if ($id > 0
+                   || ($id !== false && $id == 0 && $itemtype == "Entity")) {
                   $response = $this->getItem($itemtype, $id, $this->parameters);
                   if (isset($response['date_mod'])) {
                      $datemod = strtotime($response['date_mod']);
@@ -257,6 +276,7 @@ class APIRest extends API {
                break;
 
             case "PUT" : // update item(s)
+            case "PATCH" : // update item(s)
                if (!isset($this->parameters['input'])) {
                   $this->messageBadArrayError();
                }
@@ -419,18 +439,9 @@ class APIRest extends API {
          $parameters['upload_result'] = [];
          $parameters['input']->_filename = [];
          $parameters['input']->_prefix_filename = [];
-         foreach ($_FILES as $filename => $files) {
-            $upload_result
-               = GLPIUploadHandler::uploadFiles(['name'           => $filename,
-                                                 'print_response' => false]);
-            foreach ($upload_result as $uresult) {
-               $parameters['input']->_filename[] = $uresult[0]->name;
-               $parameters['input']->_prefix_filename[] = $uresult[0]->prefix;
-            }
-            $parameters['upload_result'][] = $upload_result;
-         }
 
       } else if (strpos($content_type, "application/x-www-form-urlencoded") !== false) {
+         /** @var array $postvars */
          parse_str($body, $postvars);
          foreach ($postvars as $field => $value) {
             $parameters[$field] = $value;
@@ -446,6 +457,13 @@ class APIRest extends API {
       if (function_exists('getallheaders')) {
          //apache specific
          $headers = getallheaders();
+         if (false !== $headers && count($headers) > 0) {
+            $fixedHeaders = [];
+            foreach ($headers as $key => $value) {
+               $fixedHeaders[ucwords(strtolower($key), '-')] = $value;
+            }
+            $headers = $fixedHeaders;
+         }
       } else {
          // other servers
          foreach ($_SERVER as $server_key => $server_value) {
@@ -483,6 +501,16 @@ class APIRest extends API {
          $parameters['app_token'] = $headers['App-Token'];
       }
 
+      // check boolean parameters
+      foreach ($parameters as $key => &$parameter) {
+         if ($parameter === "true") {
+            $parameter = true;
+         }
+         if ($parameter === "false") {
+            $parameter = false;
+         }
+      }
+
       $this->parameters = $parameters;
 
       return "";
