@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 if (!defined('GLPI_ROOT')) {
    die("Sorry. You can't access this file directly");
 }
@@ -41,7 +37,7 @@ if (!defined('GLPI_ROOT')) {
 /**
  * CommonITILValidation Class
  *
- * @since version 0.85
+ * @since 0.85
 **/
 abstract class CommonITILValidation  extends CommonDBChild {
 
@@ -164,18 +160,23 @@ abstract class CommonITILValidation  extends CommonDBChild {
 
 
    /**
-    * @param $items_id ID of the item
+    * @param integer $items_id ID of the item
    **/
    static function canValidate($items_id) {
       global $DB;
 
-      $query = "SELECT `users_id_validate`
-                FROM `".static::getTable()."`
-                WHERE `".static::$items_id."` = '$items_id'
-                      AND `users_id_validate` = '".Session::getLoginUserID()."'";
-      $result = $DB->query($query);
+      $iterator = $DB->request([
+         'SELECT' => ['users_id_validate'],
+         'FROM'   => static::getTable(),
+         'WHERE'  => [
+            static::$items_id    => $items_id,
+            'users_id_validate'  => Session::getLoginUserID()
+         ],
+         'START'  => 0,
+         'LIMIT'  => 1
+      ]);
 
-      if ($DB->numrows($result)) {
+      if (count($iterator) > 0) {
          return true;
       }
       return false;
@@ -199,10 +200,10 @@ abstract class CommonITILValidation  extends CommonDBChild {
       if (!$hidetab) {
          $nb = 0;
          if ($_SESSION['glpishow_count_on_tabs']) {
-            $restrict = "`".static::$items_id."` = '".$item->getID()."'";
+            $restrict = [static::$items_id => $item->getID()];
             // No rights for create only count asign ones
             if (!Session::haveRightsOr(static::$rightname, static::getCreateRights())) {
-               $restrict .= " AND `users_id_validate` = '".Session::getLoginUserID()."'";
+               $restrict['users_id_validate'] = Session::getLoginUserID();
             }
             $nb = countElementsInTable(static::getTable(), $restrict);
          }
@@ -261,8 +262,10 @@ abstract class CommonITILValidation  extends CommonDBChild {
          // Set global validation to waiting
          if (($item->fields['global_validation'] == self::ACCEPTED)
              || ($item->fields['global_validation'] == self::NONE)) {
-            $input['id']                = $this->fields[static::$items_id];
-            $input['global_validation'] = self::WAITING;
+            $input = [
+               'id'                => $this->fields[static::$items_id],
+               'global_validation' => self::WAITING,
+            ];
 
             // to fix lastupdater
             if (isset($this->input['_auto_update'])) {
@@ -289,10 +292,16 @@ abstract class CommonITILValidation  extends CommonDBChild {
             $user->getFromDB($this->fields["users_id_validate"]);
             $email   = $user->getDefaultEmail();
             if (!empty($email)) {
-               $message[] = sprintf(__('Approval request send to %s'), $user->getName());
+               Session::addMessageAfterRedirect(sprintf(__('Approval request send to %s'), $user->getName()));
             } else {
-               $error[] = sprintf(__('The selected user (%s) has no valid email address. The request has been created, without email confirmation.'),
-                                  $user->getName());
+               Session::addMessageAfterRedirect(
+                  sprintf(
+                     __('The selected user (%s) has no valid email address. The request has been created, without email confirmation.'),
+                     $user->getName()
+                  ),
+                  false,
+                  ERROR
+               );
             }
          }
       }
@@ -327,7 +336,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
       }
 
       if (count($forbid_fields)) {
-         foreach ($forbid_fields as $key => $val) {
+         foreach (array_keys($forbid_fields) as $key) {
             if (isset($input[$key])) {
                unset($input[$key]);
             }
@@ -358,8 +367,10 @@ abstract class CommonITILValidation  extends CommonDBChild {
          if (($item->fields['global_validation'] == self::WAITING)
              && in_array("status", $this->updates)) {
 
-            $input['id']                = $this->fields[static::$items_id];
-            $input['global_validation'] = self::computeValidationStatus($item);
+            $input = [
+               'id'                => $this->fields[static::$items_id],
+               'global_validation' => self::computeValidationStatus($item),
+            ];
             $item->update($input);
          }
       }
@@ -372,8 +383,10 @@ abstract class CommonITILValidation  extends CommonDBChild {
       if ($item->getFromDB($this->fields[static::$items_id])) {
          if (($item->fields['global_validation'] == self::WAITING)) {
 
-            $input['id']                = $this->fields[static::$items_id];
-            $input['global_validation'] = self::NONE;
+            $input = [
+               'id'                => $this->fields[static::$items_id],
+               'global_validation' => self::NONE,
+            ];
             $item->update($input);
          }
       }
@@ -428,7 +441,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
     * @param $global             boolean (true for global status, with "no validation" option)
     *                                    (false by default)
     *
-    * @return an array
+    * @return array
    **/
    static function getAllStatusArray($withmetaforsearch = false, $global = false) {
 
@@ -453,21 +466,24 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Dropdown of validation status
     *
-    * @param $name          select name
-    * @param $options array of possible options:
+    * @param string $name    select name
+    * @param array  $options possible options:
     *      - value    : default value (default waiting)
     *      - all      : boolean display all (default false)
     *      - global   : for global validation (default false)
     *      - display  : boolean display or get string ? (default true)
     *
-    * @return nothing (display)
+    * @return string|integer Output string if display option is set to false,
+    *                        otherwise random part of dropdown id
    **/
    static function dropdownStatus($name, $options = []) {
 
-      $p['value']    = self::WAITING;
-      $p['global']   = false;
-      $p['all']      = false;
-      $p['display']  = true;
+      $p = [
+         'value'    => self::WAITING,
+         'global'   => false,
+         'all'      => false,
+         'display'  => true,
+      ];
 
       if (is_array($options) && count($options)) {
          foreach ($options as $key => $val) {
@@ -486,7 +502,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Get Ticket validation status Name
     *
-    * @param $value status ID
+    * @param integer $value status ID
    **/
    static function getStatus($value) {
 
@@ -499,7 +515,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Get Ticket validation status Color
     *
-    * @param $value status ID
+    * @param integer $value status ID
    **/
    static function getStatusColor($value) {
 
@@ -523,43 +539,6 @@ abstract class CommonITILValidation  extends CommonDBChild {
    }
 
 
-   /**
-    * All validations requests for an item have the same status ?
-    *
-    * @param $items_id   integer  item ID
-   **/
-   static function isAllValidationsHaveSameStatusForTicket($items_id) {
-      global $DB;
-
-      $query = "SELECT DISTINCT `status`
-                FROM `".static::getTable()."`
-                WHERE `".static::$items_id."` = '$items_id'";
-      $result = $DB->query($query);
-
-      return ($DB->numrows($result) == 1);
-   }
-
-
-   /**
-    * Get item validation demands count
-    *
-    * @param $items_id   integer  item ID
-   **/
-   static function getNumberValidationForTicket($items_id) {
-      global $DB;
-
-      $query = "SELECT COUNT(`id`) AS 'total'
-                FROM `".static::getTable()."`
-                WHERE `".static::$items_id."` = '$items_id'";
-
-      $result = $DB->query($query);
-      if ($DB->numrows($result)) {
-         return $DB->result($result, 0, "total");
-      }
-      return false;
-   }
-
-
    /**
     * Get item validation demands count for a user
     *
@@ -568,38 +547,38 @@ abstract class CommonITILValidation  extends CommonDBChild {
    static function getNumberToValidate($users_id) {
       global $DB;
 
-      $query = "SELECT COUNT(`id`) AS 'total'
-                FROM `".static::getTable()."`
-                WHERE `status` = '".self::WAITING."'
-                      AND `users_id_validate` = '$users_id'";
+      $row = $DB->request([
+         'FROM'   => static::getTable(),
+         'COUNT'  => 'cpt',
+         'WHERE'  => [
+            'status'             => self::WAITING,
+            'users_id_validate'  => $users_id
+         ]
+      ])->next();
 
-      $result = $DB->query($query);
-      if ($DB->numrows($result)) {
-         return $DB->result($result, 0, "total");
-      }
-      return false;
+      return $row['cpt'];
    }
 
 
    /**
     * Get the number of validations attached to an item having a specified status
     *
-    * @param $items_id   integer  item ID
-    * @param $status              status
+    * @param integer $items_id item ID
+    * @param integer $status   status
    **/
    static function getTicketStatusNumber($items_id, $status) {
       global $DB;
 
-      $query = "SELECT COUNT(`status`) AS 'total'
-                FROM `".static::getTable()."`
-                WHERE `".static::$items_id."` = '$items_id'
-                      AND `status` = '".$status."'";
+      $row = $DB->request([
+         'FROM'   => static::getTable(),
+         'COUNT'  => 'cpt',
+         'WHERE'  => [
+            static::$items_id => $items_id,
+            'status'          => $status
+         ]
+      ])->next();
 
-      $result = $DB->query($query);
-      if ($DB->numrows($result)) {
-         return $DB->result($result, 0, "total");
-      }
-      return false;
+      return $row['cpt'];
    }
 
 
@@ -609,20 +588,24 @@ abstract class CommonITILValidation  extends CommonDBChild {
     * @param $items_id   integer  item ID
     * @param $users_id   integer  user ID
     *
-    * @since version 0.85
+    * @since 0.85
     *
     * @return boolean
    **/
    static function alreadyExists($items_id, $users_id) {
       global $DB;
 
-      $query = "SELECT *
-                FROM `".static::getTable()."`
-                WHERE `".static::$items_id."` = '$items_id'
-                      AND `users_id_validate` = '".$users_id."'";
+      $iterator = $DB->request([
+         'FROM'   => static::getTable(),
+         'WHERE'  => [
+            static::$items_id    => $items_id,
+            'users_id_validate'  => $users_id
+         ],
+         'START'  => 0,
+         'LIMIT'  => 1
+      ]);
 
-      $result = $DB->query($query);
-      if ($DB->numrows($result)) {
+      if (count($iterator) > 0) {
          return true;
       }
       return false;
@@ -657,7 +640,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
 
 
    /**
-    * @since version 0.85
+    * @since 0.85
     *
     * @see CommonDBTM::showMassiveActionsSubForm()
    **/
@@ -674,7 +657,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
 
 
    /**
-    * @since version 0.85
+    * @since 0.85
     *
     * @see CommonDBTM::processMassiveActionsForOneItemtype()
    **/
@@ -726,9 +709,9 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Print the validation list into item
     *
-    * @param $item class
+    * @param CommonDBTM $item
    **/
-   function showSummary($item) {
+   function showSummary(CommonDBTM $item) {
       global $DB, $CFG_GLPI;
 
       if (!Session::haveRightsOr(static::$rightname,
@@ -745,8 +728,8 @@ abstract class CommonITILValidation  extends CommonDBChild {
       $rand   = mt_rand();
 
       if ($canadd) {
-         echo "<form method='post' name=form action='".
-                Toolbox::getItemTypeFormURL(static::$itemtype)."'>";
+         $itemtype = static::$itemtype;
+         echo "<form method='post' name=form action='".$itemtype::getFormURL()."'>";
       }
       echo "<table class='tab_cadre_fixe'>";
       echo "<tr>";
@@ -811,14 +794,11 @@ abstract class CommonITILValidation  extends CommonDBChild {
          echo "</script>\n";
       }
 
-      $query = "SELECT *
-                FROM `".$this->getTable()."`
-                WHERE `".static::$items_id."` = '".$item->getField('id')."'";
-
-      $query .= " ORDER BY submission_date DESC";
-
-      $result = $DB->query($query);
-      $number = $DB->numrows($result);
+      $iterator = $DB->Request([
+         'FROM'   => $this->getTable(),
+         'WHERE'  => [static::$items_id => $item->getField('id')],
+         'ORDER'  => 'submission_date DESC'
+      ]);
 
       $colonnes = [_x('item', 'State'), __('Request date'), __('Approval requester'),
                      __('Request comments'), __('Approval status'),
@@ -837,7 +817,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
                echo __('Send an approval request')."</a></td></tr>\n";
          }
       }
-      if ($number) {
+      if (count($iterator)) {
          $header = "<tr>";
          foreach ($colonnes as $colonne) {
             $header .= "<th>".$colonne."</th>";
@@ -850,7 +830,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
                                         sprintf(__('%1$s = %2$s'), $item->getTypeName(1),
                                                 $item->fields["name"]));
 
-         while ($row = $DB->fetch_assoc($result)) {
+         while ($row = $iterator->next()) {
             $canedit = $this->canEdit($row["id"]);
             Session::addToNavigateListItems($this->getType(), $row["id"]);
             $bgcolor = self::getStatusColor($row['status']);
@@ -903,7 +883,6 @@ abstract class CommonITILValidation  extends CommonDBChild {
     * @param $options   array    options used
     **/
    function showForm($ID, $options = []) {
-      global $CFG_GLPI;
 
       if ($ID > 0) {
          $this->canEdit($ID);
@@ -1019,7 +998,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
    }
 
 
-   function getSearchOptionsNew() {
+   function rawSearchOptions() {
       $tab = [];
 
       $tab[] = [
@@ -1097,7 +1076,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
    }
 
 
-   static function getSearchOptionsToAddNew() {
+   static function rawSearchOptionsToAdd() {
       $tab = [];
 
       $tab[] = [
@@ -1304,18 +1283,20 @@ abstract class CommonITILValidation  extends CommonDBChild {
     *  - users_id_validate       : ID of user validator
     *  - applyto
     *
-    * @return nothing (display)
+    * @return void Output is printed
    **/
    static function dropdownValidator(array $options = []) {
       global $CFG_GLPI;
 
-      $params['name']               = '';
-      $params['id']                 = 0;
-      $params['entity']             = $_SESSION['glpiactive_entity'];
-      $params['right']              = ['validate_request', 'validate_incident'];
-      $params['groups_id']          = 0;
-      $params['users_id_validate']  = [];
-      $params['applyto']            = 'show_validator_field';
+      $params = [
+        'name'              => '' ,
+        'id'                => 0,
+        'entity'            => $_SESSION['glpiactive_entity'],
+        'right'             => ['validate_request', 'validate_incident'],
+        'groups_id'         => 0,
+        'users_id_validate' => [],
+        'applyto'           => 'show_validator_field',
+      ];
 
       foreach ($options as $key => $val) {
          $params[$key] = $val;
@@ -1363,7 +1344,9 @@ abstract class CommonITILValidation  extends CommonDBChild {
    static function getGroupUserHaveRights(array $options = []) {
       global $DB;
 
-      $params['entity']    = $_SESSION['glpiactive_entity'];
+      $params = [
+         'entity' => $_SESSION['glpiactive_entity'],
+      ];
       if (static::$itemtype == 'Ticket') {
          $params['right']  = ['validate_request', 'validate_incident'];
       } else {
@@ -1376,14 +1359,14 @@ abstract class CommonITILValidation  extends CommonDBChild {
       }
 
       $list       = [];
-      $restrict   = "";
+      $restrict   = [];
 
-      $res = User::getSqlSearchResult (false, $params['right'], $params['entity']);
-      while ($data = $DB->fetch_assoc($res)) {
+      $res = User::getSqlSearchResult(false, $params['right'], $params['entity']);
+      while ($data = $res->next()) {
          $list[] = $data['id'];
       }
       if (count($list) > 0) {
-         $restrict = "`glpi_users`.`id` IN ('".implode("', '", $list)."') ";
+         $restrict = ['glpi_users.id' => $list];
       }
       $users = Group_user::getGroupUsers($params['groups_id'], $restrict);
 
@@ -1396,23 +1379,23 @@ abstract class CommonITILValidation  extends CommonDBChild {
     *
     * @param $item CommonITILObject
     *
-    * @return validation status
+    * @return integer
    **/
    static function computeValidationStatus(CommonITILObject $item) {
 
       $validation_status  = self::WAITING;
 
-      $accepted           = 0;
-      $rejected           = 0;
-
       // Percent of validation
       $validation_percent = $item->fields['validation_percent'];
 
       $statuses           = [self::ACCEPTED => 0,
                                   self::WAITING  => 0,
                                   self::REFUSED  => 0];
-      $restrict           = "`".static::$items_id."` = '".$item->getID()."'";
-      $validations        = getAllDatasFromTable(static::getTable(), $restrict);
+      $validations        = getAllDatasFromTable(
+         static::getTable(), [
+            static::$items_id => $item->getID()
+         ]
+      );
 
       if ($total = count($validations)) {
          foreach ($validations as $validation) {
@@ -1441,9 +1424,9 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Get the validation statistics
     *
-    * @param $tID tickets id
+    * @param integer $tID tickets id
     *
-    * @return statistics array
+    * @return string
    **/
    static function getValidationStats($tID) {
 
@@ -1452,7 +1435,7 @@ abstract class CommonITILValidation  extends CommonDBChild {
       $nb  = countElementsInTable(static::getTable(), [static::$items_id => $tID]);
 
       $stats = [];
-      foreach ($tab as $status => $name) {
+      foreach (array_keys($tab) as $status) {
          $validations = countElementsInTable(static::getTable(), [static::$items_id => $tID,
                                                                  'status'          => $status]);
          if ($validations > 0) {
@@ -1490,31 +1473,35 @@ abstract class CommonITILValidation  extends CommonDBChild {
 
       switch ($type) {
          case 'status' :
-            Html::scriptStart();
-            echo "$('[name=\"status\"]').change(function() {
-                     var status_ko = 0;
-                     var input_status = $(this).val();
-                     if (input_status != undefined) {
-                        if ((";
+            $jsScript = "
+               $(document).ready(
+                  function() {
+                     $('[name=\"status\"]').change(function() {
+                        var status_ko = 0;
+                        var input_status = $(this).val();
+                        if (input_status != undefined) {
+                           if ((";
             $first = true;
             foreach ($status as $val) {
                if (!$first) {
-                  echo "||";
+                  $jsScript .= "||";
                }
-               echo "input_status == $val";
+               $jsScript .= "input_status == $val";
                $first = false;
             }
-            echo "           )
+            $jsScript .= "           )
                                  && input_status != ".$item->fields['status']."){
-                           status_ko = 1;
+                              status_ko = 1;
+                           }
                         }
-                     }
-                     if ((status_ko == 1)
-                         && ('".$item->fields['global_validation']."' == '".self::WAITING."')) {
-                        alert('".$message."');
-                     }
-                  });";
-            echo Html::scriptEnd();
+                        if ((status_ko == 1)
+                            && ('".$item->fields['global_validation']."' == '".self::WAITING."')) {
+                           alert('".$message."');
+                        }
+                     });
+                  }
+               );";
+            echo Html::scriptBlock($jsScript);
             break;
 
          case 'solution' :
@@ -1530,9 +1517,9 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Get the ITIL object can validation status list
     *
-    * @since version 0.85
+    * @since 0.85
     *
-    * @return an array
+    * @return array
     **/
    static function getCanValidationStatusArray() {
       return [self::NONE, self::ACCEPTED];
@@ -1542,9 +1529,9 @@ abstract class CommonITILValidation  extends CommonDBChild {
    /**
     * Get the ITIL object all validation status list
     *
-    * @since version 0.85
+    * @since 0.85
     *
-    * @return an array
+    * @return array
     **/
    static function getAllValidationStatusArray() {
       return [self::NONE, self::WAITING, self::REFUSED, self::ACCEPTED];
