@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 if (!defined('GLPI_ROOT')) {
    die("Sorry. You can't access this file directly");
 }
@@ -47,6 +43,7 @@ if (!defined('GLPI_ROOT')) {
 class ConsumableItem extends CommonDBTM {
    // From CommonDBTM
    static protected $forward_entity_to = ['Consumable', 'Infocom'];
+   public $dohistory                   = true;
    protected $usenotepad               = true;
 
    static $rightname                   = 'consumable';
@@ -57,21 +54,11 @@ class ConsumableItem extends CommonDBTM {
    }
 
 
-   /**
-    * @see CommonGLPI::getMenuName()
-    *
-    * @since version 0.85
-   **/
    static function getMenuName() {
       return Consumable::getTypeName(Session::getPluralNumber());
    }
 
 
-   /**
-    * @see CommonGLPI::getAdditionalMenuLinks()
-    *
-    * @since version 0.85
-   **/
    static function getAdditionalMenuLinks() {
 
       if (static::canView()) {
@@ -81,11 +68,6 @@ class ConsumableItem extends CommonDBTM {
    }
 
 
-   /**
-    * @since version 0.84
-    *
-    * @see CommonDBTM::getPostAdditionalInfosForName
-   **/
    function getPostAdditionalInfosForName() {
 
       if (isset($this->fields["ref"]) && !empty($this->fields["ref"])) {
@@ -97,11 +79,15 @@ class ConsumableItem extends CommonDBTM {
 
    function cleanDBonPurge() {
 
-      $class = new Consumable();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
+      $this->deleteChildrenAndRelationsFromDb(
+         [
+            Consumable::class,
+         ]
+      );
 
-      $class = new Alert();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
+      // Alert does not extends CommonDBConnexity
+      $alert = new Alert();
+      $alert->cleanDBonItemDelete($this->getType(), $this->fields['id']);
    }
 
 
@@ -122,6 +108,7 @@ class ConsumableItem extends CommonDBTM {
       $this->addStandardTab('Document_Item', $ong, $options);
       $this->addStandardTab('Link', $ong, $options);
       $this->addStandardTab('Notepad', $ong, $options);
+      $this->addStandardTab('Log', $ong, $options);
 
       return $ong;
    }
@@ -130,16 +117,14 @@ class ConsumableItem extends CommonDBTM {
    /**
     * Print the consumable type form
     *
-    * @param $ID        integer ID of the item
-    * @param $options   array
-    *     - target filename : where to go when done.
-    *     - withtemplate boolean : template or basic item
+    * @param integer $ID    ID of the item
+    * @param array $options
+    *    - target filename : where to go when done.
+    *    - withtemplate boolean : template or basic item
     *
-    * @return Nothing (display)
-    *
-    **/
+    * @return true
+    */
    function showForm($ID, $options = []) {
-      global $CFG_GLPI;
 
       $this->initForm($ID, $options);
       $this->showFormHeader($options);
@@ -172,18 +157,20 @@ class ConsumableItem extends CommonDBTM {
                            'right'  => 'own_ticket',
                            'entity' => $this->fields["entities_id"]]);
       echo "</td>";
-      echo "<td rowspan='4' class='middle'>".__('Comments')."</td>";
-      echo "<td class='middle' rowspan='4'>
+      echo "<td rowspan='5' class='middle'>".__('Comments')."</td>";
+      echo "<td class='middle' rowspan='5'>
              <textarea cols='45' rows='9' name='comment' >".$this->fields["comment"]."</textarea>";
       echo "</td></tr>";
 
       echo "<tr class='tab_bg_1'>";
       echo "<td>".__('Group in charge of the hardware')."</td>";
       echo "<td>";
-      Group::dropdown(['name'      => 'groups_id_tech',
-                            'value'     => $this->fields['groups_id_tech'],
-                            'entity'    => $this->fields['entities_id'],
-                            'condition' => '`is_assign`']);
+      Group::dropdown([
+         'name'      => 'groups_id_tech',
+         'value'     => $this->fields['groups_id_tech'],
+         'entity'    => $this->fields['entities_id'],
+         'condition' => ['is_assign' => 1]
+      ]);
       echo "</td></tr>\n";
 
       echo "<tr class='tab_bg_1'>";
@@ -205,28 +192,28 @@ class ConsumableItem extends CommonDBTM {
       Alert::displayLastAlert('ConsumableItem', $ID);
       echo "</td></tr>";
 
+      echo "<tr>";
+      $tplmark = $this->getAutofillMark('otherserial', $options);
+      echo "<td>".sprintf(__('%1$s%2$s'), __('Inventory number'), $tplmark)."</td>";
+      echo "<td>";
+      $objectName = autoName(
+         $this->fields["otherserial"],
+         "otherserial",
+         (isset($options['withtemplate']) && ($options['withtemplate'] == 2)),
+         $this->getType(),
+         $this->fields["entities_id"]
+      );
+      Html::autocompletionTextField($this, 'otherserial', ['value' => $objectName]);
+      echo "</td>";
+      echo "</tr>";
+
       $this->showFormButtons($options);
 
       return true;
    }
 
 
-   /**
-    * @see CommonDBTM::getSpecificMassiveActions()
-   **/
-   function getSpecificMassiveActions($checkitem = null) {
-
-      $isadmin = static::canUpdate();
-      $actions = parent::getSpecificMassiveActions($checkitem);
-
-      if ($isadmin) {
-         MassiveAction::getAddTransferList($actions);
-      }
-
-      return $actions;
-   }
-
-   function getSearchOptionsNew() {
+   function rawSearchOptions() {
       $tab = [];
 
       $tab[] = [
@@ -260,6 +247,14 @@ class ConsumableItem extends CommonDBTM {
          'datatype'           => 'string'
       ];
 
+      $tab[] = [
+         'id'                 => '6',
+         'table'              => $this->getTable(),
+         'field'              => 'otherserial',
+         'name'               => __('Inventory number'),
+         'datatype'           => 'string'
+      ];
+
       $tab[] = [
          'id'                 => '4',
          'table'              => 'glpi_consumableitemtypes',
@@ -319,7 +314,7 @@ class ConsumableItem extends CommonDBTM {
          ]
       ];
 
-      $tab = array_merge($tab, Location::getSearchOptionsToAddNew());
+      $tab = array_merge($tab, Location::rawSearchOptionsToAdd());
 
       $tab[] = [
          'id'                 => '24',
@@ -337,7 +332,7 @@ class ConsumableItem extends CommonDBTM {
          'field'              => 'completename',
          'linkfield'          => 'groups_id_tech',
          'name'               => __('Group in charge of the hardware'),
-         'condition'          => '`is_assign`',
+         'condition'          => ['is_assign' => 1],
          'datatype'           => 'dropdown'
       ];
 
@@ -370,9 +365,9 @@ class ConsumableItem extends CommonDBTM {
       ];
 
       // add objectlock search options
-      $tab = array_merge($tab, ObjectLock::getSearchOptionsToAddNew(get_class($this)));
+      $tab = array_merge($tab, ObjectLock::rawSearchOptionsToAdd(get_class($this)));
 
-      $tab = array_merge($tab, Notepad::getSearchOptionsToAddNew());
+      $tab = array_merge($tab, Notepad::rawSearchOptionsToAdd());
 
       return $tab;
    }
@@ -386,11 +381,11 @@ class ConsumableItem extends CommonDBTM {
    /**
     * Cron action on consumables : alert if a stock is behind the threshold
     *
-    * @param $task   to log, if NULL display (default NULL)
+    * @param CronTask|null $task to log, if NULL display (default NULL)
     *
-    * @return 0 : nothing to do 1 : done with success
+    * @return integer 0 : nothing to do 1 : done with success
    **/
-   static function cronConsumable($task = null) {
+   static function cronConsumable(CronTask $task = null) {
       global $DB, $CFG_GLPI;
 
       $cron_status = 1;
@@ -402,26 +397,45 @@ class ConsumableItem extends CommonDBTM {
 
          foreach (Entity::getEntitiesToNotify('consumables_alert_repeat') as $entity => $repeat) {
 
-            $query_alert = "SELECT `glpi_consumableitems`.`id` AS consID,
-                                   `glpi_consumableitems`.`entities_id` AS entity,
-                                   `glpi_consumableitems`.`ref` AS ref,
-                                   `glpi_consumableitems`.`name` AS name,
-                                   `glpi_consumableitems`.`alarm_threshold` AS threshold,
-                                   `glpi_alerts`.`id` AS alertID,
-                                   `glpi_alerts`.`date`
-                            FROM `glpi_consumableitems`
-                            LEFT JOIN `glpi_alerts`
-                                 ON (`glpi_consumableitems`.`id` = `glpi_alerts`.`items_id`
-                                     AND `glpi_alerts`.`itemtype`='ConsumableItem')
-                            WHERE `glpi_consumableitems`.`is_deleted` = '0'
-                                  AND `glpi_consumableitems`.`alarm_threshold` >= '0'
-                                  AND `glpi_consumableitems`.`entities_id` = '".$entity."'
-                                  AND (`glpi_alerts`.`date` IS NULL
-                                       OR (`glpi_alerts`.date+$repeat) < CURRENT_TIMESTAMP());";
+            $alerts_result = $DB->request(
+               [
+                  'SELECT'    => [
+                     'glpi_consumableitems.id AS consID',
+                     'glpi_consumableitems.entities_id AS entity',
+                     'glpi_consumableitems.ref AS ref',
+                     'glpi_consumableitems.name AS name',
+                     'glpi_consumableitems.alarm_threshold AS threshold',
+                     'glpi_alerts.id AS alertID',
+                     'glpi_alerts.date',
+                  ],
+                  'FROM'      => ConsumableItem::getTable(),
+                  'LEFT JOIN' => [
+                     'glpi_alerts' => [
+                        'FKEY' => [
+                           'glpi_alerts'         => 'items_id',
+                           'glpi_consumableitems' => 'id',
+                           [
+                              'AND' => ['glpi_alerts.itemtype' => 'ConsumableItem'],
+                           ],
+                        ]
+                     ]
+                  ],
+                  'WHERE'     => [
+                     'glpi_consumableitems.is_deleted'      => 0,
+                     'glpi_consumableitems.alarm_threshold' => ['>=', 0],
+                     'glpi_consumableitems.entities_id'     => $entity,
+                     'OR'                                  => [
+                        ['glpi_alerts.date' => null],
+                        ['glpi_alerts.date' => ['<', new QueryExpression('CURRENT_TIMESTAMP() - INTERVAL ' . $repeat . ' second')]],
+                     ],
+                  ],
+               ]
+            );
+
             $message = "";
             $items   = [];
 
-            foreach ($DB->request($query_alert) as $consumable) {
+            foreach ($alerts_result as $consumable) {
                if (($unused=Consumable::getUnusedNumber($consumable["consID"]))
                               <=$consumable["threshold"]) {
                   // define message alert
@@ -440,8 +454,10 @@ class ConsumableItem extends CommonDBTM {
             }
 
             if (!empty($items)) {
-               $options['entities_id'] = $entity;
-               $options['items']       = $items;
+               $options = [
+                  'entities_id' => $entity,
+                  'items'       => $items,
+               ];
 
                if (NotificationEvent::raiseEvent('alert', new ConsumableItem(), $options)) {
                   if ($task) {
@@ -454,8 +470,10 @@ class ConsumableItem extends CommonDBTM {
                                                       " :  $message");
                   }
 
-                  $input["type"]     = Alert::THRESHOLD;
-                  $input["itemtype"] = 'ConsumableItem';
+                  $input = [
+                     'type'     => Alert::THRESHOLD,
+                     'itemtype' => 'ConsumableItem',
+                  ];
 
                   // add alerts
                   foreach ($items as $ID=>$consumable) {
@@ -505,20 +523,9 @@ class ConsumableItem extends CommonDBTM {
    }
 
 
-   /**
-    * Have I the right to "update" the Object
-    *
-    * Default is true and check entity if the objet is entity assign
-    *
-    * May be overloaded if needed
-    *
-    * Overriden here to check entities recursively
-    *
-    * @return booleen
-   **/
    function canUpdateItem() {
 
-      if (!$this->checkEntity(true)) {
+      if (!$this->checkEntity(true)) { //check entities recursively
          return false;
       }
       return true;
