@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,10 +30,6 @@
  * ---------------------------------------------------------------------
  */
 
-/** @file
-* @brief
-*/
-
 if (!defined('GLPI_ROOT')) {
    die("Sorry. You can't access this file directly");
 }
@@ -68,17 +64,17 @@ class Contract extends CommonDBTM {
 
    function cleanDBonPurge() {
 
-      $class = new Contract_Supplier();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
+      $this->deleteChildrenAndRelationsFromDb(
+         [
+            Contract_Item::class,
+            Contract_Supplier::class,
+            ContractCost::class,
+         ]
+      );
 
-      $class = new ContractCost();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
-
-      $class = new Contract_Item();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
-
-      $class = new Alert();
-      $class->cleanDBonItemDelete($this->getType(), $this->fields['id']);
+      // Alert does not extends CommonDBConnexity
+      $alert = new Alert();
+      $alert->cleanDBonItemDelete($this->getType(), $this->fields['id']);
    }
 
 
@@ -101,18 +97,25 @@ class Contract extends CommonDBTM {
    /**
     * Duplicate all contracts from a item template to his clone
     *
-    * @since version 9.2
+    * @since 9.2
     *
     * @param string $itemtype      itemtype of the item
     * @param integer $oldid        ID of the item to clone
     * @param integer $newid        ID of the item cloned
     **/
-   static function cloneItem ($itemtype, $oldid, $newid) {
+   static function cloneItem($itemtype, $oldid, $newid) {
       global $DB;
 
-      foreach ($DB->request('glpi_contracts_items',
-                            ['WHERE'  => "`items_id` = '$oldid'
-                                          AND `itemtype` = '$itemtype'"]) as $data) {
+      $result = $DB->request(
+         [
+            'FROM'   => Contract_Item::getTable(),
+            'WHERE'  => [
+               'items_id' => $oldid,
+               'itemtype' => $itemtype,
+            ],
+         ]
+      );
+      foreach ($result as $data) {
          $cd = new Contract_Item();
          unset($data['id']);
          $data['items_id'] = $newid;
@@ -122,11 +125,7 @@ class Contract extends CommonDBTM {
       }
    }
 
-   /**
-    * @since version 0.83.3
-    *
-    * @see CommonDBTM::prepareInputForAdd()
-    */
+
    function prepareInputForAdd($input) {
 
       if (isset($input["id"]) && $input["id"]>0) {
@@ -139,11 +138,7 @@ class Contract extends CommonDBTM {
    }
 
 
-   /**
-    * @since version 0.84
-   **/
    function post_addItem() {
-      global $DB;
 
       // Manage add from template
       if (isset($this->input["_oldID"])) {
@@ -362,7 +357,7 @@ class Contract extends CommonDBTM {
    }
 
 
-   static function getSearchOptionsToAddNew() {
+   static function rawSearchOptionsToAdd() {
       $tab = [];
 
       $joinparams = [
@@ -405,7 +400,7 @@ class Contract extends CommonDBTM {
          'id'                 => '29',
          'table'              => 'glpi_contracts',
          'field'              => 'name',
-         'name'               => self::getTypeName(1),
+         'name'               => __('Name'),
          'forcegroupby'       => true,
          'datatype'           => 'itemlink',
          'massiveaction'      => false,
@@ -416,7 +411,7 @@ class Contract extends CommonDBTM {
          'id'                 => '30',
          'table'              => 'glpi_contracts',
          'field'              => 'num',
-         'name'               => __('Contract number'),
+         'name'               => __('Number'),
          'forcegroupby'       => true,
          'massiveaction'      => false,
          'joinparams'         => $joinparams,
@@ -427,7 +422,7 @@ class Contract extends CommonDBTM {
          'id'                 => '129',
          'table'              => 'glpi_contracttypes',
          'field'              => 'name',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Type')),
+         'name'               => __('Type'),
          'datatype'           => 'dropdown',
          'massiveaction'      => false,
          'joinparams'         => [
@@ -442,7 +437,7 @@ class Contract extends CommonDBTM {
          'id'                 => '130',
          'table'              => 'glpi_contracts',
          'field'              => 'duration',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Duration')),
+         'name'               => __('Duration'),
          'datatype'           => 'number',
          'max'                => '120',
          'unit'               => 'month',
@@ -456,7 +451,7 @@ class Contract extends CommonDBTM {
          'table'              => 'glpi_contracts',
          'field'              => 'periodicity',
                                  //TRANS: %1$s is Contract, %2$s is field name
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Periodicity')),
+         'name'               => __('Periodicity'),
          'forcegroupby'       => true,
          'massiveaction'      => false,
          'joinparams'         => $joinparams,
@@ -478,7 +473,7 @@ class Contract extends CommonDBTM {
          'id'                 => '132',
          'table'              => 'glpi_contracts',
          'field'              => 'begin_date',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Start date')),
+         'name'               => __('Start date'),
          'forcegroupby'       => true,
          'datatype'           => 'date',
          'massiveaction'      => false,
@@ -489,7 +484,7 @@ class Contract extends CommonDBTM {
          'id'                 => '133',
          'table'              => 'glpi_contracts',
          'field'              => 'accounting_number',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Account number')),
+         'name'               => __('Account number'),
          'forcegroupby'       => true,
          'massiveaction'      => false,
          'datatype'           => 'string',
@@ -500,7 +495,7 @@ class Contract extends CommonDBTM {
          'id'                 => '134',
          'table'              => 'glpi_contracts',
          'field'              => 'end_date',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('End date')),
+         'name'               => __('End date'),
          'forcegroupby'       => true,
          'datatype'           => 'date_delay',
          'datafields'         => [
@@ -517,7 +512,7 @@ class Contract extends CommonDBTM {
          'id'                 => '135',
          'table'              => 'glpi_contracts',
          'field'              => 'notice',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Notice')),
+         'name'               => __('Notice'),
          'datatype'           => 'number',
          'max'                => '120',
          'unit'               => 'month',
@@ -530,7 +525,7 @@ class Contract extends CommonDBTM {
          'id'                 => '136',
          'table'              => 'glpi_contractcosts',
          'field'              => 'totalcost',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Cost')),
+         'name'               => __('Cost'),
          'forcegroupby'       => true,
          'usehaving'          => true,
          'datatype'           => 'decimal',
@@ -543,7 +538,7 @@ class Contract extends CommonDBTM {
          'id'                 => '137',
          'table'              => 'glpi_contracts',
          'field'              => 'billing',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Invoice period')),
+         'name'               => __('Invoice period'),
          'forcegroupby'       => true,
          'massiveaction'      => false,
          'joinparams'         => $joinparams,
@@ -565,7 +560,7 @@ class Contract extends CommonDBTM {
          'id'                 => '138',
          'table'              => 'glpi_contracts',
          'field'              => 'renewal',
-         'name'               => sprintf(__('%1$s - %2$s'), __('Contract'), __('Renewal')),
+         'name'               => __('Renewal'),
          'forcegroupby'       => true,
          'massiveaction'      => false,
          'joinparams'         => $joinparams,
@@ -576,9 +571,6 @@ class Contract extends CommonDBTM {
    }
 
 
-   /**
-    * @see CommonDBTM::getSpecificMassiveActions()
-    **/
    function getSpecificMassiveActions($checkitem = null) {
 
       $isadmin = static::canUpdate();
@@ -590,22 +582,10 @@ class Contract extends CommonDBTM {
          $actions[$prefix.'remove'] = _x('button', 'Remove an item');
       }
 
-      if ($isadmin) {
-         MassiveAction::getAddTransferList($actions);
-      }
-
       return $actions;
    }
 
 
-   /**
-    * @since version 0.84
-    *
-    * @param $field
-    * @param $name            (default '')
-    * @param $values          (default '')
-    * @param $options   array
-   **/
    static function getSpecificValueToSelect($field, $name = '', $values = '', array $options = []) {
 
       if (!is_array($values)) {
@@ -626,13 +606,6 @@ class Contract extends CommonDBTM {
    }
 
 
-   /**
-    * @since version 0.84
-    *
-    * @param $field
-    * @param $values
-    * @param $options   array
-   **/
    static function getSpecificValueToDisplay($field, $values, array $options = []) {
 
       if (!is_array($values)) {
@@ -649,7 +622,7 @@ class Contract extends CommonDBTM {
    }
 
 
-   function getSearchOptionsNew() {
+   function rawSearchOptions() {
       $tab = [];
 
       $tab[] = [
@@ -913,9 +886,9 @@ class Contract extends CommonDBTM {
       ];
 
       // add objectlock search options
-      $tab = array_merge($tab, ObjectLock::getSearchOptionsToAddNew(get_class($this)));
+      $tab = array_merge($tab, ObjectLock::rawSearchOptionsToAdd(get_class($this)));
 
-      $tab = array_merge($tab, Notepad::getSearchOptionsToAddNew());
+      $tab = array_merge($tab, Notepad::rawSearchOptionsToAdd());
 
       $tab[] = [
          'id'                 => 'cost',
@@ -1016,13 +989,13 @@ class Contract extends CommonDBTM {
     * Show central contract resume
     * HTML array
     *
-    * @return Nothing (display)
+    * @return void
     **/
    static function showCentral() {
       global $DB,$CFG_GLPI;
 
       if (!Contract::canView()) {
-         return false;
+         return;
       }
 
       // No recursive contract, not in local management
@@ -1098,19 +1071,25 @@ class Contract extends CommonDBTM {
              self::getTypeName(1)."</a></th></tr>";
 
       echo "<tr class='tab_bg_2'>";
-      $options['reset'] = 'reset';
-      $options['sort']  = 12;
-      $options['order'] = 'DESC';
-      $options['start'] = 0;
-
-      $options['criteria'][0] = ['field'      => 12,
-                                      'value'      => '<0',
-                                      'searchtype' => 'contains'];
-      $options['criteria'][1] = ['field'      => 12,
-                                      'link'       => 'AND',
-                                      'value'      => '>-30',
-                                      'searchtype' => 'contains'];
-
+      $options = [
+         'reset' => 'reset',
+         'sort'  => 12,
+         'order' => 'DESC',
+         'start' => 0,
+         'criteria' => [
+            [
+               'field'      => 12,
+               'value'      => '<0',
+               'searchtype' => 'contains',
+            ],
+            [
+               'field'      => 12,
+               'link'       => 'AND',
+               'value'      => '>-30',
+               'searchtype' => 'contains',
+            ]
+         ]
+      ];
       echo "<td><a href=\"".$CFG_GLPI["root_doc"]."/front/contract.php?".
                  Toolbox::append_params($options, '&amp;')."\">".
                  __('Contracts expired in the last 30 days')."</a> </td>";
@@ -1162,14 +1141,21 @@ class Contract extends CommonDBTM {
    function getSuppliersNames() {
       global $DB;
 
-      $query = "SELECT `glpi_suppliers`.`id`
-                FROM `glpi_contracts_suppliers`,
-                     `glpi_suppliers`
-                WHERE `glpi_contracts_suppliers`.`suppliers_id` = `glpi_suppliers`.`id`
-                      AND `glpi_contracts_suppliers`.`contracts_id` = '".$this->fields['id']."'";
-      $result = $DB->query($query);
+      $iterator = $DB->request([
+         'SELECT'       => 'glpi_suppliers.id',
+         'FROM'         => 'glpi_suppliers',
+         'INNER JOIN'   => [
+            'glpi_contracts_suppliers' => [
+               'ON' => [
+                  'glpi_contracts_suppliers' => 'suppliers_id',
+                  'glpi_suppliers'           => 'id'
+               ]
+            ]
+         ],
+         'WHERE'        => ['contracts_id' => $this->fields['id']]
+      ]);
       $out    = "";
-      while ($data = $DB->fetch_assoc($result)) {
+      while ($data = $iterator->next()) {
          $out .= Dropdown::getDropdownName("glpi_suppliers", $data['id'])."<br>";
       }
       return $out;
@@ -1184,9 +1170,11 @@ class Contract extends CommonDBTM {
    /**
     * Cron action on contracts : alert depending of the config : on notice and expire
     *
-    * @param $task for log, if NULL display (default NULL)
+    * @param CronTask $task for log, if NULL display (default NULL)
+    *
+    * @return integer
    **/
-   static function cronContract($task = null) {
+   static function cronContract(CronTask $task = null) {
       global $DB, $CFG_GLPI;
 
       if (!$CFG_GLPI["use_notifications"]) {
@@ -1194,58 +1182,130 @@ class Contract extends CommonDBTM {
       }
 
       $message       = [];
-      $items_notice  = [];
-      $items_end     = [];
       $cron_status   = 0;
 
-      $contract_infos[Alert::END]    = [];
-      $contract_infos[Alert::NOTICE] = [];
-      $contract_messages             = [];
+      $contract_infos    = [
+         Alert::END    => [],
+         Alert::NOTICE => [],
+      ];
+      $contract_messages = [];
 
       foreach (Entity::getEntitiesToNotify('use_contracts_alert') as $entity => $value) {
          $before       = Entity::getUsedConfig('send_contracts_alert_before_delay', $entity);
-         $query_notice = "SELECT `glpi_contracts`.*
-                          FROM `glpi_contracts`
-                          LEFT JOIN `glpi_alerts`
-                              ON (`glpi_contracts`.`id` = `glpi_alerts`.`items_id`
-                                  AND `glpi_alerts`.`itemtype` = 'Contract'
-                                  AND `glpi_alerts`.`type`='".Alert::NOTICE."')
-                          WHERE (`glpi_contracts`.`alert` & ".pow(2, Alert::NOTICE).") >'0'
-                                AND `glpi_contracts`.`is_deleted` = '0'
-                                AND `glpi_contracts`.`begin_date` IS NOT NULL
-                                AND `glpi_contracts`.`duration` <> '0'
-                                AND `glpi_contracts`.`notice` <> '0'
-                                AND DATEDIFF(ADDDATE(`glpi_contracts`.`begin_date`,
-                                                     INTERVAL `glpi_contracts`.`duration` MONTH),
-                                             CURDATE()) > '0'
-                                AND DATEDIFF(ADDDATE(`glpi_contracts`.`begin_date`,
-                                                     INTERVAL (`glpi_contracts`.`duration`
-                                                                -`glpi_contracts`.`notice`) MONTH),
-                                             CURDATE()) < '$before'
-                                AND `glpi_alerts`.`date` IS NULL
-                                AND `glpi_contracts`.`entities_id` = '".$entity."'";
 
-         $query_end = "SELECT `glpi_contracts`.*
-                       FROM `glpi_contracts`
-                       LEFT JOIN `glpi_alerts`
-                           ON (`glpi_contracts`.`id` = `glpi_alerts`.`items_id`
-                               AND `glpi_alerts`.`itemtype` = 'Contract'
-                               AND `glpi_alerts`.`type`='".Alert::END."')
-                       WHERE (`glpi_contracts`.`alert` & ".pow(2, Alert::END).") > '0'
-                             AND `glpi_contracts`.`is_deleted` = '0'
-                             AND `glpi_contracts`.`begin_date` IS NOT NULL
-                             AND `glpi_contracts`.`duration` <> '0'
-                             AND DATEDIFF(ADDDATE(`glpi_contracts`.`begin_date`,
-                                                  INTERVAL (`glpi_contracts`.`duration`) MONTH),
-                                          CURDATE()) < '$before'
-                             AND `glpi_alerts`.`date` IS NULL
-                             AND `glpi_contracts`.`entities_id` = '".$entity."'";
+         $query_notice = [
+            'SELECT'    => [
+               'glpi_contracts.*',
+            ],
+            'FROM'      => self::getTable(),
+            'LEFT JOIN' => [
+               'glpi_alerts' => [
+                  'FKEY' => [
+                     'glpi_alerts'    => 'items_id',
+                     'glpi_contracts' => 'id',
+                     [
+                        'AND' => [
+                           'glpi_alerts.itemtype' => 'Contract',
+                           'glpi_alerts.type'     => Alert::NOTICE,
+                        ],
+                     ],
+                  ]
+               ]
+            ],
+            'WHERE'     => [
+               [
+                  'RAW' => [
+                     DBmysql::quoteName('glpi_contracts.alert') . ' & ' . pow(2, Alert::NOTICE) => ['>', 0]
+                  ]
+               ],
+               'glpi_alerts.date'           => null,
+               'glpi_contracts.is_deleted'  => 0,
+               [
+                  'NOT' => ['glpi_contracts.begin_date' => null],
+               ],
+               'glpi_contracts.duration'    => ['!=', 0],
+               'glpi_contracts.notice'      => ['!=', 0],
+               'glpi_contracts.entities_id' => $entity,
+               [
+                  'RAW' => [
+                     'DATEDIFF(
+                         ADDDATE(
+                            ' . DBmysql::quoteName('glpi_contracts.begin_date') . ',
+                            INTERVAL ' . DBmysql::quoteName('glpi_contracts.duration') . ' MONTH
+                         ),
+                         CURDATE()
+                      )' => ['>', 0]
+                  ]
+               ],
+               [
+                  'RAW' => [
+                     'DATEDIFF(
+                         ADDDATE(
+                            ' . DBmysql::quoteName('glpi_contracts.begin_date') . ',
+                            INTERVAL (
+                               ' . DBmysql::quoteName('glpi_contracts.duration') . '
+                               - ' . DBmysql::quoteName('glpi_contracts.notice') . '
+                            ) MONTH
+                         ),
+                         CURDATE()
+                      )' => ['<', $before]
+                  ]
+               ],
+            ],
+         ];
+
+         $query_end = [
+            'SELECT'    => [
+               'glpi_contracts.*',
+            ],
+            'FROM'      => self::getTable(),
+            'LEFT JOIN' => [
+               'glpi_alerts' => [
+                  'FKEY' => [
+                     'glpi_alerts'    => 'items_id',
+                     'glpi_contracts' => 'id',
+                     [
+                        'AND' => [
+                           'glpi_alerts.itemtype' => 'Contract',
+                           'glpi_alerts.type'     => Alert::END,
+                        ],
+                     ],
+                  ]
+               ]
+            ],
+            'WHERE'     => [
+               [
+                  'RAW' => [
+                     DBmysql::quoteName('glpi_contracts.alert') . ' & ' . pow(2, Alert::END) => ['>', 0]
+                  ]
+               ],
+               'glpi_alerts.date'           => null,
+               'glpi_contracts.is_deleted'  => 0,
+               [
+                  'NOT' => ['glpi_contracts.begin_date' => null],
+               ],
+               'glpi_contracts.duration'    => ['!=', 0],
+               'glpi_contracts.entities_id' => $entity,
+               [
+                  'RAW' => [
+                     'DATEDIFF(
+                         ADDDATE(
+                            ' . DBmysql::quoteName('glpi_contracts.begin_date') . ',
+                            INTERVAL ' . DBmysql::quoteName('glpi_contracts.duration') . ' MONTH
+                         ),
+                         CURDATE()
+                      )' => ['<', $before]
+                  ]
+               ],
+            ],
+         ];
 
          $querys = ['notice' => $query_notice,
                          'end'    => $query_end];
 
          foreach ($querys as $type => $query) {
-            foreach ($DB->request($query) as $data) {
+            $result = $DB->request($query);
+            foreach ($result as $data) {
                $entity  = $data['entities_id'];
 
                $message = sprintf(__('%1$s: %2$s')."<br>\n", $data["name"],
@@ -1271,73 +1331,76 @@ class Contract extends CommonDBTM {
          }
 
          // Get contrats with periodicity alerts
-         $query_periodicity = "SELECT `glpi_contracts`.*
-                               FROM `glpi_contracts`
-                               WHERE `glpi_contracts`.`alert` & ".pow(2, Alert::PERIODICITY)." > '0'
-                                     AND `glpi_contracts`.`entities_id` = '".$entity."' ";
+         $valPow = pow(2, Alert::PERIODICITY);
+         $query_periodicity = ['FROM' => 'glpi_contracts',
+             'WHERE' => ['alert' => ['&', $valPow],
+                 'entities_id' => $entity,
+                 'is_deleted' => 0
+             ]
+         ];
 
          // Foreach ones :
          foreach ($DB->request($query_periodicity) as $data) {
-            $entity    = $data['entities_id'];
-            // Compute end date + 12 month : do not send alerts after
-            $end_alert = date('Y-m-d',
-                              strtotime($data['begin_date']." +".($data['duration']+12)." month"));
-            if (!empty($data['begin_date'])
-                && $data['periodicity']
-                && ($end_alert > date('Y-m-d'))) {
+            $entity = $data['entities_id'];
+
+            // For contracts with begin date and periodicity
+            if (!empty($data['begin_date']) && $data['periodicity']) {
                $todo = ['periodicity' => Alert::PERIODICITY];
-               if ($data['alert']&pow(2, Alert::NOTICE)) {
+               if ($data['alert'] & pow(2, Alert::NOTICE)) {
                   $todo['periodicitynotice'] = Alert::NOTICE;
                }
 
-               // Get previous alerts
+               // For the todo...
                foreach ($todo as $type => $event) {
-                  $previous_alerts[$type] = Alert::getAlertDate(__CLASS__, $data['id'], $event);
-               }
-               // compute next alert date based on already send alerts (or not)
-               foreach ($todo as $type => $event) {
-                  $next_alerts[$type] = date('Y-m-d',
-                                             strtotime($data['begin_date']." -".($before)." day"));
-                  if ($type == Alert::NOTICE) {
-                     $next_alerts[$type]
-                           = date('Y-m-d',
-                                  strtotime($next_alerts[$type]." -".($data['notice'])." month"));
+                  /**
+                   * Previous alert
+                   */
+                  // Get previous alerts from DB
+                  $previous_alert = [
+                     $type => Alert::getAlertDate(__CLASS__, $data['id'], $event),
+                  ];
+                  // If alert never occurs...
+                  if (empty($previous_alert[$type])) {
+                     // We define it a long time ago [in a galaxy far, far away... ;-)]
+                     $previous_alert[$type] = date('Y-m-d', 0);
                   }
 
-                  $today_limit = date('Y-m-d',
-                                      strtotime(date('Y-m-d')." -" .($data['periodicity'])." month"));
-
-                  // Init previous by begin date if not set
-                  if (empty($previous_alerts[$type])) {
-                     $previous_alerts[$type] = $today_limit;
+                  /**
+                   * Next alert
+                   */
+                  // Computation of first alert : Contract [begin date + periodicity] - Config [alert xxx days before]
+                  $next_alert = [
+                     $type => date('Y-m-d', strtotime($data['begin_date'] . " +" . $data['periodicity'] . " month -" . ($before) . " day")),
+                  ];
+                  // If a notice is defined
+                  if ($event == Alert::NOTICE) {
+                     // Will decrease of the Contract notice duration
+                     $next_alert[$type] = date('Y-m-d', strtotime($next_alert[$type] . " -" . ($data['notice']) . " month"));
                   }
 
-                  while (($next_alerts[$type] < $previous_alerts[$type])
-                         && ($next_alerts[$type] < $end_alert)) {
-                     $next_alerts[$type]
-                        = date('Y-m-d',
-                               strtotime($next_alerts[$type]." +".($data['periodicity'])." month"));
+                  // Computation of contract renewal
+                  while ($next_alert[$type] < $previous_alert[$type]) {
+                     // Increasing of Contract periodicity...
+                     $next_alert[$type] = date('Y-m-d', strtotime($next_alert[$type] . " +" . ($data['periodicity']) . " month"));
                   }
 
                   // If this date is passed : clean alerts and send again
-                  if ($next_alerts[$type] <= date('Y-m-d')) {
-                     $alert              = new Alert();
+                  if ($next_alert[$type] <= date('Y-m-d')) {
+                     $alert = new Alert();
                      $alert->clear(__CLASS__, $data['id'], $event);
-                     $real_alert_date    = date('Y-m-d',
-                                                strtotime($next_alerts[$type]." +".($before)." day"));
-                     $message            = sprintf(__('%1$s: %2$s')."<br>\n",
-                                                 $data["name"], Html::convDate($real_alert_date));
+                     // Computation of the real date => add Config [alert xxx days before]
+                     $real_alert_date = date('Y-m-d', strtotime($next_alert[$type] . " +" . ($before) . " day"));
+                     $message = sprintf(__('%1$s: %2$s') . "<br>\n", $data["name"], Html::convDate($real_alert_date));
                      $data['alert_date'] = $real_alert_date;
                      $contract_infos[$type][$entity][$data['id']] = $data;
 
                      switch ($type) {
                         case 'periodicitynotice' :
-                           $contract_messages[$type][$entity]
-                                 = __('Contract entered in notice time for period')."<br>";
+                           $contract_messages[$type][$entity] = __('Contract entered in notice time for period') . "<br>";
                            break;
 
                         case 'periodicity' :
-                           $contract_messages[$type][$entity] = __('Contract period ended')."<br>";
+                           $contract_messages[$type][$entity] = __('Contract period ended') . "<br>";
                            break;
                      }
                      $contract_messages[$type][$entity] .= $message;
@@ -1347,13 +1410,13 @@ class Contract extends CommonDBTM {
          }
       }
       foreach (['notice'            => Alert::NOTICE,
-                     'end'               => Alert::END,
-                     'periodicity'       => Alert::PERIODICITY,
-                     'periodicitynotice' => Alert::NOTICE] as $event => $type ) {
+                  'end'               => Alert::END,
+                  'periodicity'       => Alert::PERIODICITY,
+                  'periodicitynotice' => Alert::NOTICE] as $event => $type ) {
          if (isset($contract_infos[$event]) && count($contract_infos[$event])) {
             foreach ($contract_infos[$event] as $entity => $contracts) {
                if (NotificationEvent::raiseEvent($event, new self(),
-                                                 ['entities_id' => $entity,
+                               ['entities_id' => $entity,
                                                        'items'       => $contracts])) {
                   $message     = $contract_messages[$event][$entity];
                   $cron_status = 1;
@@ -1366,16 +1429,17 @@ class Contract extends CommonDBTM {
                                                               $entityname, $message));
                   }
 
-                  $alert               = new Alert();
-                  $input["itemtype"]   = __CLASS__;
-                  $input["type"]       = $type;
+                  $alert = new Alert();
+                  $input = [
+                     'itemtype' => __CLASS__,
+                     'type'     => $type,
+                  ];
                   foreach ($contracts as $id => $contract) {
                      $input["items_id"] = $id;
 
                      $alert->add($input);
                      unset($alert->fields['id']);
                   }
-
                } else {
                   $entityname = Dropdown::getDropdownName('glpi_entities', $entity);
                   //TRANS: %1$s is entity name, %2$s is the message
@@ -1398,7 +1462,7 @@ class Contract extends CommonDBTM {
     * Print a select with contracts
     *
     * Print a select named $name with contracts options and selected value $value
-    * @param $options   array of possible options:
+    * @param array $options
     *    - name          : string / name of the select (default is contracts_id)
     *    - value         : integer / preselected value (default 0)
     *    - entity        : integer or array / restrict to a defined entity or array of entities
@@ -1412,22 +1476,24 @@ class Contract extends CommonDBTM {
     *    - display       : boolean / display or return string (default true)
     *    - expired       : boolean / display expired contract (default false)
     *
-    * @return Nothing (display)
+    * @return string|integer HTML output, or random part of dropdown ID.
    **/
    static function dropdown($options = []) {
       global $DB;
 
       //$name,$entity_restrict=-1,$alreadyused=array(),$nochecklimit=false
-      $p['name']           = 'contracts_id';
-      $p['value']          = '';
-      $p['entity']         = '';
-      $p['rand']           = mt_rand();
-      $p['entity_sons']    = false;
-      $p['used']           = [];
-      $p['nochecklimit']   = false;
-      $p['on_change']      = '';
-      $p['display']        = true;
-      $p['expired']        = false;
+      $p = [
+         'name'           => 'contracts_id',
+         'value'          => '',
+         'entity'         => '',
+         'rand'           => mt_rand(),
+         'entity_sons'    => false,
+         'used'           => [],
+         'nochecklimit'   => false,
+         'on_change'      => '',
+         'display'        => true,
+         'expired'        => false,
+      ];
 
       if (is_array($options) && count($options)) {
          foreach ($options as $key => $val) {
@@ -1466,7 +1532,7 @@ class Contract extends CommonDBTM {
       $query = "SELECT `glpi_contracts`.*
                 FROM `glpi_contracts`
                 LEFT JOIN `glpi_entities` ON (`glpi_contracts`.`entities_id` = `glpi_entities`.`id`)
-                WHERE `glpi_contracts`.`is_deleted` = '0' AND `glpi_contracts`.`is_template` = '0'
+                WHERE `glpi_contracts`.`is_deleted` = 0 AND `glpi_contracts`.`is_template` = 0
                 $entrest $idrest $expired
                 ORDER BY `glpi_entities`.`completename`,
                          `glpi_contracts`.`name` ASC,
@@ -1510,18 +1576,20 @@ class Contract extends CommonDBTM {
     *
     * Print a select named $name with contract renewal options and selected value $value
     *
-    * @param $name      string   HTML select name
-    * @param $value     integer  HTML select selected value (default = 0)
-    * @param $display   boolean  get or display string ? (true by default)
+    * @param string  $name    HTML select name
+    * @param integer $value   HTML select selected value (default = 0)
+    * @param boolean $display get or display string ? (true by default)
     *
-    * @return Nothing (display)
+    * @return string|integer HTML output, or random part of dropdown ID.
    **/
    static function dropdownContractRenewal($name, $value = 0, $display = true) {
 
-      $tmp[0] = __('Never');
-      $tmp[1] = __('Tacit');
-      $tmp[2] = __('Express');
-      return Dropdown::showFromArray($name, $tmp, ['value'   => $value,
+      $values = [
+         __('Never'),
+         __('Tacit'),
+         __('Express'),
+      ];
+      return Dropdown::showFromArray($name, $values, ['value'   => $value,
                                                         'display' => $display]);
    }
 
@@ -1554,9 +1622,9 @@ class Contract extends CommonDBTM {
    /**
     * Get renewal ID by name
     *
-    * @param $value the name of the renewal
+    * @param string $value the name of the renewal
     *
-    * @return the ID of the renewal
+    * @return integer ID of the renewal
    **/
    static function getContractRenewalIDByName($value) {
 
@@ -1571,14 +1639,18 @@ class Contract extends CommonDBTM {
 
 
    /**
-    * @param $options array
+    * @param array $options
+    *
+    * @return string|integer HTML output, or random part of dropdown ID.
    **/
    static function dropdownAlert(array $options) {
 
-      $p['name']           = 'alert';
-      $p['value']          = 0;
-      $p['display']        = true;
-      $p['inherit_parent'] = false;
+      $p = [
+         'name'           => 'alert',
+         'value'          => 0,
+         'display'        => true,
+         'inherit_parent' => false,
+      ];
 
       if (count($options)) {
          foreach ($options as $key => $val) {
@@ -1600,29 +1672,31 @@ class Contract extends CommonDBTM {
    /**
     * Get the possible value for contract alert
     *
-    * @since version 0.83
+    * @since 0.83
     *
-    * @param $val if not set, ask for all values, else for 1 value (default NULL)
+    * @param string|integer|null $val if not set, ask for all values, else for 1 value (default NULL)
     *
-    * @return array or string
+    * @return string|string[]
    **/
    static function getAlertName($val = null) {
 
-      $tmp[0]                                                  = Dropdown::EMPTY_VALUE;
-      $tmp[pow(2, Alert::END)]                                 = __('End');
-      $tmp[pow(2, Alert::NOTICE)]                              = __('Notice');
-      $tmp[(pow(2, Alert::END) + pow(2, Alert::NOTICE))]       = __('End + Notice');
-      $tmp[pow(2, Alert::PERIODICITY)]                         = __('Period end');
-      $tmp[pow(2, Alert::PERIODICITY) + pow(2, Alert::NOTICE)] = __('Period end + Notice');
+      $names = [
+         0                                                  => Dropdown::EMPTY_VALUE,
+         pow(2, Alert::END)                                 => __('End'),
+         pow(2, Alert::NOTICE)                              => __('Notice'),
+         (pow(2, Alert::END) + pow(2, Alert::NOTICE))       => __('End + Notice'),
+         pow(2, Alert::PERIODICITY)                         => __('Period end'),
+         pow(2, Alert::PERIODICITY) + pow(2, Alert::NOTICE) => __('Period end + Notice'),
+      ];
 
       if (is_null($val)) {
-         return $tmp;
+         return $names;
       }
       // Default value for display
-      $tmp[0] = ' ';
+      $names[0] = ' ';
 
-      if (isset($tmp[$val])) {
-         return $tmp[$val];
+      if (isset($names[$val])) {
+         return $names[$val];
       }
       // If not set and is a string return value
       if (is_string($val)) {
@@ -1637,9 +1711,11 @@ class Contract extends CommonDBTM {
    **/
    function showDebug() {
 
-      $options['entities_id'] = $this->getEntityID();
-      $options['contracts']   = [];
-      $options['items']       = [];
+      $options = [
+         'entities_id' => $this->getEntityID(),
+         'contracts'   => [],
+         'items'       => [],
+      ];
       NotificationEvent::debugEvent($this, $options);
    }
 
@@ -1653,12 +1729,6 @@ class Contract extends CommonDBTM {
    }
 
 
-
-   /**
-    * @since version 0.85
-    *
-    * @see CommonDBTM::getMassiveActionsForItemtype()
-   **/
    static function getMassiveActionsForItemtype(array &$actions, $itemtype, $is_deleted = 0,
                                                 CommonDBTM $checkitem = null) {
       global $CFG_GLPI;
