@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -48,7 +48,7 @@ class DBmysql {
    //! Default Database
    public $dbdefault          = "";
    //! Database Handler
-   public $dbh;
+   private $dbh;
    //! Database Error
    public $error              = 0;
 
@@ -65,9 +65,6 @@ class DBmysql {
    //to calculate execution time
    public $execution_time          = false;
 
-   //to simulate transactions (for tests)
-   public $objcreated = [];
-
    private $cache_disabled = false;
 
    /**
@@ -160,13 +157,12 @@ class DBmysql {
    function query($query) {
       global $CFG_GLPI, $DEBUG_SQL, $SQL_TOTAL_REQUEST;
 
-      if (($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE)
-          && $CFG_GLPI["debug_sql"]) {
+      $is_debug = isset($_SESSION['glpi_use_mode']) && ($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE);
+      if ($is_debug && $CFG_GLPI["debug_sql"]) {
          $SQL_TOTAL_REQUEST++;
          $DEBUG_SQL["queries"][$SQL_TOTAL_REQUEST] = $query;
       }
-      if (($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE)
-         && $CFG_GLPI["debug_sql"] || $this->execution_time === true) {
+      if ($is_debug && $CFG_GLPI["debug_sql"] || $this->execution_time === true) {
          $TIMER                                    = new Timer();
          $TIMER->start();
       }
@@ -178,19 +174,14 @@ class DBmysql {
                    $this->dbh->error."\n";
          $error .= Toolbox::backtrace(false, 'DBmysql->query()', ['Toolbox::backtrace()']);
 
-         Toolbox::logInFile("sql-errors", $error);
-         if (class_exists('GlpitestSQLError')) { // For unit test
-            throw new GlpitestSQLError($error);
-         }
+         Toolbox::logSqlError($error);
 
-         if (($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE)
-             && $CFG_GLPI["debug_sql"]) {
+         if (($is_debug || isAPI()) && $CFG_GLPI["debug_sql"]) {
             $DEBUG_SQL["errors"][$SQL_TOTAL_REQUEST] = $this->error();
          }
       }
 
-      if (($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE)
-          && $CFG_GLPI["debug_sql"]) {
+      if ($is_debug && $CFG_GLPI["debug_sql"]) {
          $TIME                                   = $TIMER->getTime();
          $DEBUG_SQL["times"][$SQL_TOTAL_REQUEST] = $TIME;
       }
@@ -201,20 +192,33 @@ class DBmysql {
    }
 
    /**
-    * Execute a MySQL query
+    * Execute a MySQL query and die
+    * (optionnaly with a message) if it fails
     *
     * @since 0.84
     *
     * @param string $query   Query to execute
-    * @param string $message Explaination of query (default '')
+    * @param string $message Explanation of query (default '')
     *
     * @return mysqli_result Query result handler
     */
    function queryOrDie($query, $message = '') {
-      //TRANS: %1$s is the description, %2$s is the query, %3$s is the error message
-      $res = $this->query($query)
-             or die(sprintf(__('%1$s - Error during the database query: %2$s - Error is %3$s'),
-                            $message, $query, $this->error()));
+      $res = $this->query($query);
+      if (!$res) {
+         //TRANS: %1$s is the description, %2$s is the query, %3$s is the error message
+         $message = sprintf(
+            __('%1$s - Error during the database query: %2$s - Error is %3$s'),
+            $message,
+            $query,
+            $this->error()
+         );
+         if (isCommandLine()) {
+            throw new \RuntimeException($message);
+         } else {
+            echo $message . "\n";
+            die(1);
+         }
+      }
       return $res;
    }
 
@@ -228,10 +232,12 @@ class DBmysql {
     * @throws GlpitestSQLError
     */
    function prepare($query) {
+      global $CFG_GLPI, $DEBUG_SQL, $SQL_TOTAL_REQUEST;
+
       $res = @$this->dbh->prepare($query);
       if (!$res) {
          // no translation for error logs
-         $error = "  *** MySQL prepare error:\n  SQL: ".addslashes($query)."\n  Error: ".
+         $error = "  *** MySQL prepare error:\n  SQL: ".$query."\n  Error: ".
                    $this->dbh->error."\n";
          $error .= Toolbox::backtrace(false, 'DBmysql->prepare()', ['Toolbox::backtrace()']);
 
@@ -240,8 +246,10 @@ class DBmysql {
             throw new GlpitestSQLError($error);
          }
 
-         if (($_SESSION['glpi_use_mode'] == Session::DEBUG_MODE)
+         if (isset($_SESSION['glpi_use_mode'])
+             && $_SESSION['glpi_use_mode'] == Session::DEBUG_MODE
              && $CFG_GLPI["debug_sql"]) {
+            $SQL_TOTAL_REQUEST++;
             $DEBUG_SQL["errors"][$SQL_TOTAL_REQUEST] = $this->error();
          }
       }
@@ -253,7 +261,7 @@ class DBmysql {
     *
     * @param mysqli_result $result MySQL result handler
     * @param int           $i      Row offset to give
-    * @param type          $field  Field to give
+    * @param string        $field  Field to give
     *
     * @return mixed Value of the Row $i and the Field $field of the Mysql $result
     */
@@ -371,19 +379,30 @@ class DBmysql {
    /**
     * List tables in database
     *
-    * @param string $table table name condition (glpi_% as default to retrieve only glpi tables)
+    * @param string $table Table name condition (glpi_% as default to retrieve only glpi tables)
+    * @param array  $where Where clause to append
     *
-    * @return mysqli_result list of tables
+    * @return DBmysqlIterator
     */
-   function list_tables($table = "glpi_%") {
-      return $this->query(
-         "SELECT TABLE_NAME FROM information_schema.`TABLES`
-             WHERE TABLE_SCHEMA = '{$this->dbdefault}'
-                AND TABLE_TYPE = 'BASE TABLE'
-                AND TABLE_NAME LIKE '$table'"
-      );
+   function listTables($table = 'glpi_%', array $where = []) {
+      $iterator = $this->request([
+         'SELECT' => 'TABLE_NAME',
+         'FROM'   => 'information_schema.TABLES',
+         'WHERE'  => [
+            'TABLE_SCHEMA' => $this->dbdefault,
+            'TABLE_TYPE'   => 'BASE TABLE',
+            'TABLE_NAME'   => ['LIKE', $table]
+         ] + $where
+      ]);
+      return $iterator;
    }
 
+   public function getMyIsamTables() {
+      $iterator = $this->listTables('glpi_%', ['engine' => 'MyIsam']);
+      return $iterator;
+   }
+
+
    /**
     * List fields of a table
     *
@@ -456,7 +475,7 @@ class DBmysql {
     * @return boolean TRUE on success or FALSE on failure.
     */
    function close() {
-      if ($this->dbh) {
+      if ($this->connected && $this->dbh) {
          return $this->dbh->close();
       }
       return false;
@@ -502,6 +521,13 @@ class DBmysql {
             if ($this->query($formattedQuerytorun)) { //if no success continue to concatenate
                $formattedQuery = "";
                $lastresult     = true;
+               if (!isCommandLine()) {
+                  // Flush will prevent proxy to timeout as it will receive data.
+                  // Flush requires a content to be sent, so we sent sp&aces as multiple spaces
+                  // will be shown as a single one on browser.
+                  echo ' ';
+                  flush();
+               }
             } else {
                $lastresult = false;
             }
@@ -539,61 +565,11 @@ class DBmysql {
     * @return DBmysqlIterator
     */
    public function request ($tableorsql, $crit = "", $debug = false) {
-      return new DBmysqlIterator($this, $tableorsql, $crit, $debug);
+      $iterator = new DBmysqlIterator($this);
+      $iterator->execute($tableorsql, $crit, $debug);
+      return $iterator;
    }
 
-    /**
-     *  Optimize sql table
-     *
-     * @var DB $DB
-     *
-     * @param mixed   $migration Migration class (default NULL)
-     * @param boolean $cron      To know if optimize must be done (false by default)
-     *
-     * @return int number of tables
-     */
-   static function optimize_tables($migration = null, $cron = false) {
-      global $DB;
-
-      $crashed_tables = self::checkForCrashedTables();
-      if (!empty($crashed_tables)) {
-         Toolbox::logDebug("Cannot launch automatic action : crashed tables detected");
-         return -1;
-      }
-
-      if (!is_null($migration) && method_exists($migration, 'displayMessage')) {
-         $migration->displayTitle(__('Optimizing tables'));
-         $migration->addNewMessageArea('optimize_table'); // to force new ajax zone
-         $migration->displayMessage(sprintf(__('%1$s - %2$s'), __('optimize'), __('Start')));
-      }
-      $result = $DB->list_tables();
-      $nb     = 0;
-
-      while ($line = $DB->fetch_row($result)) {
-         $table = $line[0];
-
-         // For big database to reduce delay of migration
-         if ($cron
-             || (countElementsInTable($table) < 15000000)) {
-
-            if (!is_null($migration) && method_exists($migration, 'displayMessage')) {
-               $migration->displayMessage(sprintf(__('%1$s - %2$s'), __('optimize'), $table));
-            }
-
-            $query = "OPTIMIZE TABLE `".$table."`;";
-            $DB->query($query);
-            $nb++;
-         }
-      }
-      $DB->free_result($result);
-
-      if (!is_null($migration)
-          && method_exists($migration, 'displayMessage') ) {
-         $migration->displayMessage(sprintf(__('%1$s - %2$s'), __('optimize'), __('End')));
-      }
-
-      return $nb;
-   }
 
    /**
     * Get information about DB connection for showSystemInformations
@@ -687,35 +663,6 @@ class DBmysql {
       return $lock_ok;
    }
 
-   /**
-   * Check for crashed MySQL Tables
-   *
-   * @since 0.90.2
-   *
-   * @var DB $DB
-    *
-   * @return string[] array with supposed crashed table and check message
-   */
-   static public function checkForCrashedTables() {
-      global $DB;
-      $crashed_tables = [];
-
-      $result_tables = $DB->list_tables();
-
-      while ($line = $DB->fetch_row($result_tables)) {
-         $query  = "CHECK TABLE `".$line[0]."` FAST";
-         $result  = $DB->query($query);
-         if ($DB->numrows($result) > 0) {
-            $row = $DB->fetch_array($result);
-            if ($row['Msg_type'] != 'status' && $row['Msg_type'] != 'note') {
-               $crashed_tables[] = ['table'    => $row[0],
-                                         'Msg_type' => $row['Msg_type'],
-                                         'Msg_text' => $row['Msg_text']];
-            }
-         }
-      }
-      return $crashed_tables;
-   }
 
    /**
     * Check if a table exists
@@ -728,17 +675,16 @@ class DBmysql {
     **/
    public function tableExists($tablename) {
       // Get a list of tables contained within the database.
-      $result = $this->list_tables("%".$tablename."%");
+      $result = $this->listTables("%$tablename%");
 
-      if ($this->numrows($result)) {
-         while ($data = $this->fetch_row($result)) {
-            if ($data[0] === $tablename) {
+      if (count($result)) {
+         while ($data = $result->next()) {
+            if ($data['TABLE_NAME'] === $tablename) {
                return true;
             }
          }
       }
 
-      $this->free_result($result);
       return false;
    }
 
@@ -776,4 +722,493 @@ class DBmysql {
    public function disableTableCaching() {
       $this->cache_disabled = true;
    }
+
+   /**
+    * Quote field name
+    *
+    * @since 9.3
+    *
+    * @param string $name of field to quote (or table.field)
+    *
+    * @return string
+    */
+   public static function quoteName($name) {
+      //handle verbatim names
+      if ($name instanceof QueryExpression) {
+         return $name->getValue();
+      }
+      //handle aliases
+      $names = preg_split('/ AS /i', $name);
+      if (count($names) > 2) {
+         throw new \RuntimeException(
+            'Invalid field name ' . $name
+         );
+      }
+      if (count($names) == 2) {
+         $name = self::quoteName($names[0]);
+         $name .= ' AS ' . self::quoteName($names[1]);
+         return $name;
+      } else {
+         if (strpos($name, '.')) {
+            $n = explode('.', $name, 2);
+            $table = self::quoteName($n[0]);
+            $field = ($n[1] === '*') ? $n[1] : self::quoteName($n[1]);
+            return "$table.$field";
+         }
+         return ($name[0]=='`' ? $name : ($name === '*') ? $name : "`$name`");
+      }
+   }
+
+   /**
+    * Quote value for insert/update
+    *
+    * @param mixed $value Value
+    *
+    * @return mixed
+    */
+   public static function quoteValue($value) {
+      if ($value instanceof QueryParam || $value instanceof QueryExpression) {
+         //no quote for query parameters nor expressions
+         $value = $value->getValue();
+      } else if ($value === null || $value === 'NULL' || $value === 'null') {
+         $value = 'NULL';
+      } else if (!preg_match("/^`.*?`$/", $value)) { //`field` is valid only for mysql :/
+         //phone numbers may start with '+' and will be considered as numeric
+         $value = "'$value'";
+      }
+      return $value;
+   }
+
+   /**
+    * Builds an insert statement
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $params Query parameters ([field name => field value)
+    *
+    * @return string
+    */
+   public function buildInsert($table, $params) {
+      $query = "INSERT INTO " . self::quoteName($table) . " (";
+
+      $fields = [];
+      foreach ($params as $key => &$value) {
+         $fields[] = $this->quoteName($key);
+         $value = $this->quoteValue($value);
+      }
+
+      $query .= implode(', ', $fields);
+      $query .= ") VALUES (";
+      $query .= implode(", ", $params);
+      $query .= ")";
+
+      return $query;
+   }
+
+   /**
+    * Insert a row in the database
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $params Query parameters ([field name => field value)
+    *
+    * @return mysqli_result|boolean Query result handler
+    */
+   public function insert($table, $params) {
+      $result = $this->query(
+         $this->buildInsert($table, $params)
+      );
+      return $result;
+   }
+
+   /**
+    * Insert a row in the database and die
+    * (optionnaly with a message) if it fails
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $params  Query parameters ([field name => field value)
+    * @param string $message Explanation of query (default '')
+    *
+    * @return mysqli_result|boolean Query result handler
+    */
+   function insertOrDie($table, $params, $message = '') {
+      $insert = $this->buildInsert($table, $params);
+      $res = $this->query($insert);
+      if (!$res) {
+         //TRANS: %1$s is the description, %2$s is the query, %3$s is the error message
+         $message = sprintf(
+            __('%1$s - Error during the database query: %2$s - Error is %3$s'),
+            $message,
+            $insert,
+            $this->error()
+         );
+         if (isCommandLine()) {
+            throw new \RuntimeException($message);
+         } else {
+            echo $message . "\n";
+            die(1);
+         }
+      }
+      return $res;
+   }
+
+   /**
+    * Builds an update statement
+    *
+    * @since 9.3
+    *
+    * @param string $table   Table name
+    * @param array  $params  Query parameters ([field name => field value)
+    * @param array  $clauses Clauses to use. If not 'WHERE' key specified, will b the WHERE clause (@see DBmysqlIterator capabilities)
+    * @param array  $joins  JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return string
+    */
+   public function buildUpdate($table, $params, $clauses, array $joins = []) {
+      //when no explicit "WHERE", we only have a WHEre clause.
+      if (!isset($clauses['WHERE'])) {
+         $clauses  = ['WHERE' => $clauses];
+      } else {
+         $known_clauses = ['WHERE', 'ORDER', 'LIMIT', 'START'];
+         foreach (array_keys($clauses) as $key) {
+            if (!in_array($key, $known_clauses)) {
+               throw new \RuntimeException(
+                  str_replace(
+                     '%clause',
+                     $key,
+                     'Trying to use an unknonw clause (%clause) building update query!'
+                  )
+               );
+            }
+         }
+      }
+
+      if (!count($clauses['WHERE'])) {
+         throw new \RuntimeException('Cannot run an UPDATE query without WHERE clause!');
+      }
+
+      $query  = "UPDATE ". self::quoteName($table);
+
+      //JOINS
+      $it = new DBmysqlIterator($this);
+      $query .= $it->analyzeJoins($joins);
+
+      $query .= " SET ";
+      foreach ($params as $field => $value) {
+         $query .= self::quoteName($field) . " = ".$this->quoteValue($value).", ";
+      }
+      $query = rtrim($query, ', ');
+
+      $query .= " WHERE " . $it->analyseCrit($clauses['WHERE']);
+
+      // ORDER BY
+      if (isset($clauses['ORDER']) && !empty($clauses['ORDER'])) {
+         $query .= $it->handleOrderClause($clauses['ORDER']);
+      }
+
+      if (isset($clauses['LIMIT']) && !empty($clauses['LIMIT'])) {
+         $offset = (isset($clauses['START']) && !empty($clauses['START'])) ? $clauses['START'] : null;
+         $query .= $it->handleLimits($clauses['LIMIT'], $offset);
+      }
+
+      return $query;
+   }
+
+   /**
+    * Update a row in the database
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $params Query parameters ([:field name => field value)
+    * @param array  $where  WHERE clause
+    * @param array  $joins  JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return mysqli_result|boolean Query result handler
+    */
+   public function update($table, $params, $where, array $joins = []) {
+      $query = $this->buildUpdate($table, $params, $where, $joins);
+      $result = $this->query($query);
+      return $result;
+   }
+
+   /**
+    * Update a row in the database or die
+    * (optionnaly with a message) if it fails
+    *
+    * @since 9.3
+    *
+    * @param string $table   Table name
+    * @param array  $params  Query parameters ([:field name => field value)
+    * @param array  $where   WHERE clause
+    * @param string $message Explanation of query (default '')
+    * @param array  $joins   JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return mysqli_result|boolean Query result handler
+    */
+   function updateOrDie($table, $params, $where, $message = '', array $joins = []) {
+      $update = $this->buildUpdate($table, $params, $where, $joins);
+      $res = $this->query($update);
+      if (!$res) {
+         //TRANS: %1$s is the description, %2$s is the query, %3$s is the error message
+         $message = sprintf(
+            __('%1$s - Error during the database query: %2$s - Error is %3$s'),
+            $message,
+            $update,
+            $this->error()
+         );
+         if (isCommandLine()) {
+            throw new \RuntimeException($message);
+         } else {
+            echo $message . "\n";
+            die(1);
+         }
+      }
+      return $res;
+   }
+
+   /**
+    * Update a row in the database or insert a new one
+    *
+    * @since 9.4
+    *
+    * @param string  $table   Table name
+    * @param array   $params  Query parameters ([:field name => field value)
+    * @param array   $where   WHERE clause
+    * @param boolean $onlyone Do the update only one one element, defaults to true
+    *
+    * @return mysqli_result|boolean Query result handler
+    */
+   public function updateOrInsert($table, $params, $where, $onlyone = true) {
+      $req = $this->request($table, $where);
+      $data = array_merge($where, $params);
+      if ($req->count() == 0) {
+         return $this->insertOrDie($table, $data, 'Unable to create new element or update existing one');
+      } else if ($req->count() == 1 || !$onlyone) {
+         return $this->updateOrDie($table, $data, $where, 'Unable to create new element or update existing one');
+      } else {
+         Toolbox::logWarning('Update would change too many rows!');
+         return false;
+      }
+   }
+
+   /**
+    * Builds a delete statement
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $params Query parameters ([field name => field value)
+    * @param array  $where  WHERE clause (@see DBmysqlIterator capabilities)
+    * @param array  $joins  JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return string
+    */
+   public function buildDelete($table, $where, array $joins = []) {
+
+      if (!count($where)) {
+         throw new \RuntimeException('Cannot run an DELETE query without WHERE clause!');
+      }
+
+      $query  = "DELETE " . self::quoteName($table) . " FROM ". self::quoteName($table);
+
+      $it = new DBmysqlIterator($this);
+      $query .= $it->analyzeJoins($joins);
+      $query .= " WHERE " . $it->analyseCrit($where);
+
+      return $query;
+   }
+
+   /**
+    * Delete rows in the database
+    *
+    * @since 9.3
+    *
+    * @param string $table  Table name
+    * @param array  $where  WHERE clause
+    * @param array  $joins  JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return mysqli_result|boolean Query result handler
+    */
+   public function delete($table, $where, array $joins = []) {
+      $query = $this->buildDelete($table, $where, $joins);
+      $result = $this->query($query);
+      return $result;
+   }
+
+   /**
+    * Delete a row in the database and die
+    * (optionnaly with a message) if it fails
+    *
+    * @since 9.3
+    *
+    * @param string $table   Table name
+    * @param array  $where   WHERE clause
+    * @param string $message Explanation of query (default '')
+    * @param array  $joins   JOINS criteria array
+    *
+    * @since 9.4.0 $joins parameter added
+    * @return mysqli_result|boolean Query result handler
+    */
+   function deleteOrDie($table, $where, $message = '', array $joins = []) {
+      $update = $this->buildDelete($table, $where, $joins);
+      $res = $this->query($update);
+      if (!$res) {
+         //TRANS: %1$s is the description, %2$s is the query, %3$s is the error message
+         $message = sprintf(
+            __('%1$s - Error during the database query: %2$s - Error is %3$s'),
+            $message,
+            $update,
+            $this->error()
+         );
+         if (isCommandLine()) {
+            throw new \RuntimeException($message);
+         } else {
+            echo $message . "\n";
+            die(1);
+         }
+
+      }
+      return $res;
+   }
+
+
+   /**
+    * Get table schema
+    *
+    * @param string $table Table name,
+    * @param string|null $structure Raw table structure
+    *
+    * @return array
+    */
+   public function getTableSchema($table, $structure = null) {
+      if ($structure === null) {
+         $structure = $this->query("SHOW CREATE TABLE `$table`")->fetch_row();
+         $structure = $structure[1];
+      }
+
+      //get table index
+      $index = preg_grep(
+         "/^\s\s+?KEY/",
+         array_map(
+            function($idx) { return rtrim($idx, ','); },
+            explode("\n", $structure)
+         )
+      );
+      //get table schema, without index, without AUTO_INCREMENT
+      $structure = preg_replace(
+         [
+            "/\s\s+KEY .*/",
+            "/AUTO_INCREMENT=\d+ /"
+         ],
+         "",
+         $structure
+      );
+      $structure = preg_replace('/,(\s)?$/m', '', $structure);
+      $structure = preg_replace('/ COMMENT \'(.+)\'/', '', $structure);
+
+      $structure = str_replace(
+         [
+            " COLLATE utf8_unicode_ci",
+            " CHARACTER SET utf8",
+            ', ',
+         ], [
+            '',
+            '',
+            ',',
+         ],
+         trim($structure)
+      );
+
+      //do not check engine nor collation
+      $structure = preg_replace(
+         '/\) ENGINE.*$/',
+         '',
+         $structure
+      );
+
+      //Mariadb 10.2 will return current_timestamp()
+      //while older retuns CURRENT_TIMESTAMP...
+      $structure = preg_replace(
+         '/ CURRENT_TIMESTAMP\(\)/i',
+         ' CURRENT_TIMESTAMP',
+         $structure
+      );
+
+      //Mariadb 10.2 allow default values on longblob, text and longtext
+      $defaults = [];
+      preg_match_all(
+         '/^.+ (longblob|text|longtext) .+$/m',
+         $structure,
+         $defaults
+      );
+      if (count($defaults[0])) {
+         foreach ($defaults[0] as $line) {
+               $structure = str_replace(
+                  $line,
+                  str_replace(' DEFAULT NULL', '', $line),
+                  $structure
+               );
+         }
+      }
+
+      $structure = preg_replace("/(DEFAULT) ([-|+]?\d+)(\.\d+)?/", "$1 '$2$3'", $structure);
+      //$structure = preg_replace("/(DEFAULT) (')?([-|+]?\d+)(\.\d+)(')?/", "$1 '$3'", $structure);
+      $structure = preg_replace('/(BIGINT)\(\d+\)/i', '$1', $structure);
+      $structure = preg_replace('/(TINYINT) /i', '$1(4) ', $structure);
+
+      return [
+         'schema' => strtolower($structure),
+         'index'  => $index
+      ];
+
+   }
+
+   /**
+    * Get database raw version
+    *
+    * @return string
+    */
+   public function getVersion() {
+      global $DB;
+      $req = $DB->request('SELECT version()')->next();
+      $raw = $req['version()'];
+      return $raw;
+   }
+
+   /**
+    * Starts a transaction
+    *
+    * @return boolean
+    */
+   public function beginTransaction() {
+      return $this->dbh->begin_transaction();
+   }
+
+   /**
+    * Commits a transaction
+    *
+    * @return boolean
+    */
+   public function commit() {
+      return $this->dbh->commit();
+   }
+
+   /**
+    * Roolbacks a transaction
+    *
+    * @return boolean
+    */
+   public function rollBack() {
+      return $this->dbh->rollback();
+   }
 }
