@@ -2,7 +2,7 @@
 /**
  * ---------------------------------------------------------------------
  * GLPI - Gestionnaire Libre de Parc Informatique
- * Copyright (C) 2015-2017 Teclib' and contributors.
+ * Copyright (C) 2015-2018 Teclib' and contributors.
  *
  * http://glpi-project.org
  *
@@ -30,12 +30,8 @@
  * ---------------------------------------------------------------------
 */
 
-/** @file
-* @brief
-*/
-
 // Current version of GLPI
-define('GLPI_VERSION', '9.2.1');
+define('GLPI_VERSION', '9.4.1.1');
 if (substr(GLPI_VERSION, -4) === '-dev') {
    //for dev version
    define('GLPI_PREVER', str_replace('-dev', '', GLPI_VERSION));
@@ -45,10 +41,10 @@ if (substr(GLPI_VERSION, -4) === '-dev') {
    );
 } else {
    //for stable version
-   define("GLPI_SCHEMA_VERSION", '9.2.1');
+   define("GLPI_SCHEMA_VERSION", '9.4.1');
 }
 define('GLPI_MIN_PHP', '5.6.0'); // Must also be changed in top of index.php
-define('GLPI_YEAR', '2017');
+define('GLPI_YEAR', '2019');
 if (!defined('GLPI_DEMO_MODE')) {
    define('GLPI_DEMO_MODE', '0');
 }
@@ -63,7 +59,9 @@ define("GLPI_CSRF_MAX_TOKENS", "100");
 
 // for compatibility with mysql 5.7
 // TODO: this var need to be set to 0 after review of all sql queries)
-define("GLPI_FORCE_EMPTY_SQL_MODE", "1");
+if (!defined('GLPI_FORCE_EMPTY_SQL_MODE')) {
+   define("GLPI_FORCE_EMPTY_SQL_MODE", "1");
+}
 
 // rights
 define("READ", 1);
@@ -77,56 +75,66 @@ define("UPDATENOTE", 64);
 define("UNLOCK", 128);
 
 // dictionnaries
-// 0 Name - 1 lang file - 2 extjs - 3 tiny_mce - 4 english lang name
-$CFG_GLPI['languages'] =  //| NAME in native lang    |LANG FILE  |jquery| tinymce|english names|standard plural number
-      ['ar_SA' => ['العَرَبِيَّةُ',        'ar_SA.mo','ar',    'ar','arabic'     , 103],
-            'bg_BG' => ['Български',            'bg_BG.mo','bg',    'bg','bulgarian'  , 2],
-            'id_ID' => ['Bahasa Indonesia',     'id_ID.mo','id',    'id','indonesian' , 2],
-            'ms_MY' => ['Bahasa Melayu',        'ms_MY.mo','ms',    'ms','malay'      , 2],
-            'ca_ES' => ['Català',               'ca_ES.mo','ca',    'ca','catalan'    , 2], // ca_CA
-            'cs_CZ' => ['Čeština',              'cs_CZ.mo','cs',    'cs','czech'      , 10],
-            'de_DE' => ['Deutsch',              'de_DE.mo','de',    'de','german'     , 2],
-            'da_DK' => ['Dansk',                'da_DK.mo','da',    'da','danish'     , 2]     , // dk_DK
-            'et_EE' => ['Eesti',                'et_EE.mo','et',    'et','estonian'   , 2], // ee_ET
-            'en_GB' => ['English',              'en_GB.mo','en-GB', 'en','english'    , 2],
-            'en_US' => ['English (US)',         'en_US.mo','en-GB', 'en','english'    , 2],
-            'es_AR' => ['Español (Argentina)',  'es_AR.mo','es',    'es','spanish'    , 2],
-            'es_CO' => ['Español (Colombia)',   'es_CO.mo','es',    'es','spanish'    , 2],
-            'es_ES' => ['Español (España)',     'es_ES.mo','es',    'es','spanish'    , 2],
-            'es_419' => ['Español (América Latina)', 'es_419.mo','es',    'es','spanish' , 2],
-            'es_MX' => ['Español (Mexico)',     'es_MX.mo','es',    'es','spanish'    , 2],
-            'es_VE' => ['Español (Venezuela)',  'es_VE.mo','es',    'es','spanish'    , 2],
-            'eu_ES' => ['Euskara',              'eu_ES.mo','eu',    'en','basque'     , 2],
-            'fr_FR' => ['Français',             'fr_FR.mo','fr',    'fr','french'     , 2],
-            'gl_ES' => ['Galego',               'gl_ES.mo','gl',    'gl','galician'   , 2],
-            'el_GR' => ['Ελληνικά',             'el_GR.mo','el',    'el','greek'      , 2], // el_EL
-            'he_IL' => ['עברית',                 'he_IL.mo','he',    'he','hebrew'     , 2], // he_HE
-            'hr_HR' => ['Hrvatski',             'hr_HR.mo','hr',    'hr','croatian'   , 2],
-            'hu_HU' => ['Magyar',               'hu_HU.mo','hu',    'hu','hungarian'  , 2],
-            'it_IT' => ['Italiano',             'it_IT.mo','it',    'it','italian'    , 2],
-            'lv_LV' => ['Latviešu',             'lv_LV.mo','lv',    'lv','latvian'    , 2],
-            'lt_LT' => ['Lietuvių',             'lt_LT.mo','lt',    'lt','lithuanian' , 2],
-            'nl_NL' => ['Nederlands',           'nl_NL.mo','nl',    'nl','dutch'      , 2],
-            'nb_NO' => ['Norsk (Bokmål)',       'nb_NO.mo','no',    'nb','norwegian'  , 2], // no_NB
-            'nn_NO' => ['Norsk (Nynorsk)',      'nn_NO.mo','no',    'nn','norwegian'  , 2], // no_NN
-            'fa_IR' => ['فارسی',                'fa_IR.mo','fa',    'fa','persian'    , 2],
-            'pl_PL' => ['Polski',               'pl_PL.mo','pl',    'pl','polish'     , 2],
-            'pt_PT' => ['Português',            'pt_PT.mo','pt',    'pt','portuguese' , 2],
-            'pt_BR' => ['Português do Brasil',  'pt_BR.mo','pt-BR', 'pt','brazilian portuguese'    , 2],
-            'ro_RO' => ['Română',               'ro_RO.mo','ro',    'en','romanian'    , 2],
-            'ru_RU' => ['Русский',              'ru_RU.mo','ru',    'ru','russian'    , 2],
-            'sk_SK' => ['Slovenčina',           'sk_SK.mo','sk',    'sk','slovak'    , 10],
-            'sl_SI' => ['Slovenščina',          'sl_SI.mo','sl',    'sl','slovenian slovene'    , 2],
-            'sr_RS' => ['Srpski',               'sr_RS.mo','sr',    'sr','serbian'    , 2],
-            'fi_FI' => ['Suomi',                'fi_FI.mo','fi',    'fi','finish'    , 2],
-            'sv_SE' => ['Svenska',              'sv_SE.mo','sv',    'sv','swedish'    , 2],
-            'vi_VN' => ['Tiếng Việt',           'vi_VN.mo','vi',    'vi','vietnamese'    , 2],
-            'th_TH' => ['ภาษาไทย',              'th_TH.mo','th',    'th','thai'    , 2],
-            'tr_TR' => ['Türkçe',               'tr_TR.mo','tr',    'tr','turkish'    , 2],
-            'uk_UA' => ['Українська',           'uk_UA.mo','uk',    'en','ukrainian'    , 2], // ua_UA
-            'ja_JP' => ['日本語',                'ja_JP.mo','ja',    'ja','japanese'    , 2],
-            'zh_CN' => ['简体中文',              'zh_CN.mo','zh-CN', 'zh','chinese'    , 2],
-            'zh_TW' => ['繁體中文',              'zh_TW.mo','zh-TW', 'zh','chinese'    , 2],];
+$CFG_GLPI['languages'] = [
+   //Code       Name in native lang          LANG FILE      jquery tinymce english name            standard plural number
+   'ar_SA'  => ['العَرَبِيَّةُ',                   'ar_SA.mo',    'ar',    'ar', 'arabic',               103],
+   'bg_BG'  => ['Български',                 'bg_BG.mo',    'bg',    'bg', 'bulgarian',            2],
+   'id_ID'  => ['Bahasa Indonesia',          'id_ID.mo',    'id',    'id', 'indonesian',           2],
+   'ms_MY'  => ['Bahasa Melayu',             'ms_MY.mo',    'ms',    'ms', 'malay',                2],
+   'ca_ES'  => ['Català',                    'ca_ES.mo',    'ca',    'ca', 'catalan',              2], // ca_CA
+   'cs_CZ'  => ['Čeština',                   'cs_CZ.mo',    'cs',    'cs', 'czech',                10],
+   'de_DE'  => ['Deutsch',                   'de_DE.mo',    'de',    'de', 'german',               2],
+   'da_DK'  => ['Dansk',                     'da_DK.mo',    'da',    'da', 'danish',               2]     , // dk_DK
+   'et_EE'  => ['Eesti',                     'et_EE.mo',    'et',    'et', 'estonian',             2], // ee_ET
+   'en_GB'  => ['English',                   'en_GB.mo',    'en-GB', 'en', 'english',              2],
+   'en_US'  => ['English (US)',              'en_US.mo',    'en-GB', 'en', 'english',              2],
+   'es_AR'  => ['Español (Argentina)',       'es_AR.mo',    'es',    'es', 'spanish',              2],
+   'es_CO'  => ['Español (Colombia)',        'es_CO.mo',    'es',    'es', 'spanish',              2],
+   'es_ES'  => ['Español (España)',          'es_ES.mo',    'es',    'es', 'spanish',              2],
+   'es_419' => ['Español (América Latina)',  'es_419.mo',   'es',    'es', 'spanish',              2],
+   'es_MX'  => ['Español (Mexico)',          'es_MX.mo',    'es',    'es', 'spanish',              2],
+   'es_VE'  => ['Español (Venezuela)',       'es_VE.mo',    'es',    'es', 'spanish',              2],
+   'eu_ES'  => ['Euskara',                   'eu_ES.mo',    'eu',    'eu', 'basque',               2],
+   'fr_FR'  => ['Français',                  'fr_FR.mo',    'fr',    'fr', 'french',               2],
+   'fr_CA'  => ['Français (Canada)',         'fr_CA.mo',    'fr',    'fr', 'french',               2],
+   'gl_ES'  => ['Galego',                    'gl_ES.mo',    'gl',    'gl', 'galician',             2],
+   'el_GR'  => ['Ελληνικά',                  'el_GR.mo',    'el',    'el', 'greek',                2], // el_EL
+   'he_IL'  => ['עברית',                     'he_IL.mo',    'he',    'he', 'hebrew',               2], // he_HE
+   'hi_IN'  => ['हिन्दी',                     'hi_IN.mo',    'hi',    'hi_IN', 'hindi' ,            2],
+   'hr_HR'  => ['Hrvatski',                  'hr_HR.mo',    'hr',    'hr', 'croatian',             2],
+   'hu_HU'  => ['Magyar',                    'hu_HU.mo',    'hu',    'hu', 'hungarian',            2],
+   'it_IT'  => ['Italiano',                  'it_IT.mo',    'it',    'it', 'italian',              2],
+   'kn'     => ['ಕನ್ನಡ',                      'kn.mo',       'en-GB', 'en', 'kannada',              2],
+   'lv_LV'  => ['Latviešu',                  'lv_LV.mo',    'lv',    'lv', 'latvian',              2],
+   'lt_LT'  => ['Lietuvių',                  'lt_LT.mo',    'lt',    'lt', 'lithuanian',           2],
+   'nl_NL'  => ['Nederlands',                'nl_NL.mo',    'nl',    'nl', 'dutch',                2],
+   'nb_NO'  => ['Norsk (Bokmål)',            'nb_NO.mo',    'no',    'nb', 'norwegian',            2], // no_NB
+   'nn_NO'  => ['Norsk (Nynorsk)',           'nn_NO.mo',    'no',    'nn', 'norwegian',            2], // no_NN
+   'fa_IR'  => ['فارسی',                     'fa_IR.mo',    'fa',    'fa', 'persian',              2],
+   'pl_PL'  => ['Polski',                    'pl_PL.mo',    'pl',    'pl', 'polish',               2],
+   'pt_PT'  => ['Português',                 'pt_PT.mo',    'pt',    'pt', 'portuguese',           2],
+   'pt_BR'  => ['Português do Brasil',       'pt_BR.mo',    'pt-BR', 'pt', 'brazilian portuguese', 2],
+   'ro_RO'  => ['Română',                    'ro_RO.mo',    'ro',    'en', 'romanian',             2],
+   'ru_RU'  => ['Русский',                   'ru_RU.mo',    'ru',    'ru', 'russian',              2],
+   'sk_SK'  => ['Slovenčina',                'sk_SK.mo',    'sk',    'sk', 'slovak',               10],
+   'sl_SI'  => ['Slovenščina',               'sl_SI.mo',    'sl',    'sl', 'slovenian slovene',    2],
+   'sr_RS'  => ['Srpski',                    'sr_RS.mo',    'sr',    'sr', 'serbian',              2],
+   'fi_FI'  => ['Suomi',                     'fi_FI.mo',    'fi',    'fi', 'finish',               2],
+   'sv_SE'  => ['Svenska',                   'sv_SE.mo',    'sv',    'sv', 'swedish',              2],
+   'vi_VN'  => ['Tiếng Việt',                'vi_VN.mo',    'vi',    'vi', 'vietnamese',           2],
+   'th_TH'  => ['ภาษาไทย',                   'th_TH.mo',    'th',    'th', 'thai',                 2],
+   'tr_TR'  => ['Türkçe',                    'tr_TR.mo',    'tr',    'tr', 'turkish',              2],
+   'uk_UA'  => ['Українська',                'uk_UA.mo',    'uk',    'en', 'ukrainian',            2], // ua_UA
+   'ja_JP'  => ['日本語',                    'ja_JP.mo',    'ja',    'ja', 'japanese',             2],
+   'zh_CN'  => ['简体中文',                  'zh_CN.mo',    'zh-CN', 'zh', 'chinese',              2],
+   'zh_TW'  => ['繁體中文',                  'zh_TW.mo',    'zh-TW', 'zh', 'chinese',              2],
+   'ko_KR'  => ['한국/韓國',                 'ko_KR.mo',    'ko',    'ko', 'korean',               1],
+   'zh_HK'  => ['香港',                      'zh_HK.mo',    'zh-HK', 'zh', 'chinese',              2],
+   'be_BY'  => ['Belarussian',               'be_BY.mo',    'be',    'be', 'belarussian',          3],
+   'is_IS'  => ['íslenska',                  'is_IS.mo',    'is',    'en', 'icelandic',            2],
+   'eo'     => ['Esperanto',                 'eo.mo',       'eo',    'en', 'esperanto',            2],
+   'es_CL'  => ['Español chileno',           'es_CL',       'es',    'es', 'spanish chilean',      2]
+];
 
 $DEFAULT_PLURAL_NUMBER = 2;
 
@@ -142,7 +150,12 @@ define("GLPIKEY", "GLPI£i'snarss'ç");
 
 //Telemetry
 if (!defined('GLPI_TELEMETRY_URI')) {
-   define('GLPI_TELEMETRY_URI', 'http://glpi-project.org/telemetry');
+   define('GLPI_TELEMETRY_URI', 'https://telemetry.glpi-project.org');
+}
+
+// GLPI Network
+if (!defined('GLPI_NETWORK_SERVICES')) {
+   define('GLPI_NETWORK_SERVICES', 'https://services.glpi-network.com');
 }
 
 // TIMES
@@ -186,11 +199,12 @@ $LANG             = [];
 $CFG_GLPI["unicity_types"]                = ['Budget', 'Computer', 'Contact', 'Contract',
                                                   'Infocom', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'Software',
-                                                  'SoftwareLicense', 'Supplier','User', 'Certicate'];
+                                                  'SoftwareLicense', 'Supplier','User', 'Certicate', 'Rack', 'Enclosure', 'PDU'];
 
 $CFG_GLPI["state_types"]                  = ['Computer', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'SoftwareLicense',
-                                                  'Certificate'];
+                                                  'Certificate', 'Enclosure', 'PDU', 'Line',
+                                                   'Rack', 'SoftwareVersion'];
 
 $CFG_GLPI["asset_types"]                  = ['Computer', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'SoftwareLicense',
@@ -228,9 +242,15 @@ $CFG_GLPI["itemdevices"]                  = ['Item_DevicePowerSupply', 'Item_Dev
 
 $CFG_GLPI["contract_types"]               = array_merge(['Computer', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'Project', 'Line',
-                                                  'Software', 'SoftwareLicense', 'Certificate'],
+                                                  'Software', 'SoftwareLicense', 'Certificate',
+                                                  'DCRoom', 'Rack', 'Enclosure'],
                                                   $CFG_GLPI['itemdevices']);
 
+$CFG_GLPI["report_types"]                 = ['Computer', 'Monitor', 'NetworkEquipment',
+                                                  'Peripheral', 'Phone', 'Printer', 'Project',
+                                                  'Software', 'SoftwareLicense', 'Certificate'];
+
+
 $CFG_GLPI["directconnect_types"]          = ['Monitor', 'Peripheral', 'Phone', 'Printer'];
 
 $CFG_GLPI["infocom_types"]                = ['Cartridge', 'CartridgeItem', 'Computer',
@@ -250,11 +270,11 @@ $CFG_GLPI["linkgroup_types"]              = ['Computer', 'Consumable', 'Monitor'
                                                   'Peripheral', 'Phone', 'Printer', 'Software',
                                                   'SoftwareLicense', 'Certificate'];
 
-$CFG_GLPI["linkuser_tech_types"]          = ['Computer', 'Monitor', 'NetworkEquipment',
+$CFG_GLPI["linkuser_tech_types"]          = ['Computer', 'ConsumableItem', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'Software',
                                                   'SoftwareLicense', 'Certificate'];
 
-$CFG_GLPI["linkgroup_tech_types"]         = ['Computer', 'Consumable', 'Monitor', 'NetworkEquipment',
+$CFG_GLPI["linkgroup_tech_types"]         = ['Computer', 'ConsumableItem', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'Software',
                                                   'SoftwareLicense', 'Certificate'];
 
@@ -267,7 +287,7 @@ $CFG_GLPI["location_types"]               = ['Budget', 'CartridgeItem', 'Consuma
 $CFG_GLPI["ticket_types"]                 = ['Computer', 'Monitor', 'NetworkEquipment',
                                                   'Peripheral', 'Phone', 'Printer', 'Software',
                                                   'SoftwareLicense', 'Certificate',
-                                                  'Line'];
+                                                  'Line', 'DCRoom', 'Rack', 'Enclosure'];
 
 $CFG_GLPI["link_types"]                   = ['Budget', 'CartridgeItem', 'Computer',
                                                   'ConsumableItem', 'Contact', 'Contract', 'Monitor',
@@ -288,7 +308,7 @@ $CFG_GLPI["dictionnary_types"]            = ['ComputerModel', 'ComputerType', 'M
 $CFG_GLPI["helpdesk_visible_types"]       = ['Software'];
 
 $CFG_GLPI["networkport_types"]            = ['Computer', 'NetworkEquipment', 'Peripheral',
-                                                  'Phone', 'Printer'];
+                                                  'Phone', 'Printer', 'Enclosure', 'PDU'];
 
 // Warning : the order is used for displaying different NetworkPort types ! Keep it !
 $CFG_GLPI['networkport_instantiations']   = ['NetworkPortEthernet', 'NetworkPortWifi' ,
@@ -304,17 +324,17 @@ $CFG_GLPI['device_types']                 = ['DeviceMotherboard', 'DeviceFirmwar
                                                   'DeviceSimcard', 'DeviceSensor'];
 
 $CFG_GLPI["itemdevices_types"]            = ['Computer', 'NetworkEquipment', 'Peripheral',
-                                                  'Phone', 'Printer'];
+                                                  'Phone', 'Printer', 'Enclosure'];
 
 $CFG_GLPI["itemdevices_itemaffinity"]     = ['Computer'];
 
 $CFG_GLPI["itemdevicememory_types"]       = ['Computer', 'NetworkEquipment', 'Peripheral', 'Printer'];
 
-$CFG_GLPI["itemdevicepowersupply_types"]  = ['Computer', 'NetworkEquipment'];
+$CFG_GLPI["itemdevicepowersupply_types"]  = ['Computer', 'NetworkEquipment', 'Enclosure'];
 
 $CFG_GLPI["itemdevicenetworkcard_types"]  = ['Computer', 'NetworkEquipment', 'Peripheral', 'Phone', 'Printer'];
 
-$CFG_GLPI['itemdeviceharddrive_types']    = ['Computer', 'Peripheral'];
+$CFG_GLPI['itemdeviceharddrive_types']    = ['Computer', 'Peripheral', 'NetworkEquipment', 'Printer'];
 
 $CFG_GLPI['itemdevicebattery_types']      = ['Computer', 'Peripheral', 'Phone', 'Printer'];
 
@@ -324,8 +344,17 @@ $CFG_GLPI['itemdevicesimcard_types']      = ['Computer', 'Peripheral', 'Phone',
 
 $CFG_GLPI['itemdevicegeneric_types']      = ['*'];
 
+$CFG_GLPI['itemdevicepci_types']          = ['*'];
+
 $CFG_GLPI['itemdevicesensor_types']       = ['Computer', 'Peripheral'];
 
+$CFG_GLPI['itemdeviceprocessor_types']    = ['Computer'];
+
+$CFG_GLPI['itemdevicesoundcard_types']    = ['Computer'];
+
+$CFG_GLPI['itemdevicegraphiccard_types']  = ['Computer'];
+
+$CFG_GLPI['itemdevicemotherboard_types']  = ['Computer'];
 
 $CFG_GLPI["notificationtemplates_types"]  = ['CartridgeItem', 'Change', 'ConsumableItem',
                                              'Contract', 'Crontask', 'DBConnection',
@@ -346,7 +375,8 @@ $CFG_GLPI["rulecollections_types"]        = ['RuleImportEntityCollection',
                                                   'RuleMailCollectorCollection',
                                                   'RuleRightCollection',
                                                   'RuleSoftwareCategoryCollection',
-                                                  'RuleTicketCollection'];
+                                                  'RuleTicketCollection',
+                                                  'RuleAssetCollection'];
 
 // Items which can planned something
 $CFG_GLPI['planning_types']               = ['ChangeTask', 'ProblemTask', 'Reminder',
@@ -354,12 +384,12 @@ $CFG_GLPI['planning_types']               = ['ChangeTask', 'ProblemTask', 'Remin
 $CFG_GLPI['planning_add_types']           = ['Reminder'];
 
 $CFG_GLPI["globalsearch_types"]           = ['Computer', 'Contact', 'Contract',
-                                                  'Document',  'Monitor',
-                                                  'NetworkEquipment', 'Peripheral', 'Phone',
-                                                  'Printer', 'Software', 'SoftwareLicense',
-                                                  'Ticket', 'Problem', 'Change',
-                                                  'User', 'Group', 'Project', 'Supplier',
-                                                  'Budget', 'Certificate'];
+                                             'Document',  'Monitor',
+                                             'NetworkEquipment', 'Peripheral', 'Phone',
+                                             'Printer', 'Software', 'SoftwareLicense',
+                                             'Ticket', 'Problem', 'Change',
+                                             'User', 'Group', 'Project', 'Supplier',
+                                             'Budget', 'Certificate', 'Line'];
 
 // New config options which can be missing during migration
 $CFG_GLPI["number_format"]  = 0;
@@ -385,8 +415,7 @@ $CFG_GLPI['user_pref_field'] = ['backcreated', 'csv_delimiter', 'date_format',
                                      'priority_6', 'refresh_ticket_list', 'set_default_tech',
                                      'set_default_requester', 'show_count_on_tabs',
                                      'show_jobs_at_login', 'task_private', 'task_state',
-                                     'use_flat_dropdowntree', 'layout', 'ticket_timeline',
-                                     'ticket_timeline_keep_replaced_tabs', 'palette',
+                                     'use_flat_dropdowntree', 'layout', 'palette',
                                      'highcontrast_css'];
 
 $CFG_GLPI['layout_excluded_pages'] = ["profile.form.php",
@@ -411,7 +440,7 @@ $CFG_GLPI['lock_lockable_objects'] = ['Budget',  'Change', 'Contact', 'Contract'
                                            'SoftwareLicense', 'Certificate'];
 
 $CFG_GLPI['inventory_lockable_objects'] = ['Computer_Item',  'Computer_SoftwareLicense',
-                                           'Computer_SoftwareVersion', 'ComputerDisk', 'ComputerVirtualMachine',
+                                           'Computer_SoftwareVersion', 'Item_Disk', 'ComputerVirtualMachine',
                                            'NetworkPort', 'NetworkName', 'IPAddress'];
 
 $CFG_GLPI["kb_types"]              = ['Budget', 'Change', 'Computer',
@@ -426,9 +455,14 @@ $CFG_GLPI["certificate_types"]     = ['Computer',
                                       'Phone', 'Printer',
                                       'SoftwareLicense', 'User'];
 
+$CFG_GLPI["rackable_types"]        = ['Computer', 'Monitor', 'NetworkEquipment', 'Peripheral', 'Enclosure', 'PDU'];
+
 $CFG_GLPI['javascript'] = [
    'central'   => [
-      'central'   => ['fullcalendar']
+      'central'   => ['fullcalendar', 'tinymce']
+   ],
+   'assets'    => [
+      'rack'         => ['colorpicker', 'gridstack']
    ],
    'helpdesk'  => [
       'planning'  => ['fullcalendar', 'colorpicker', 'tinymce'],
@@ -439,9 +473,14 @@ $CFG_GLPI['javascript'] = [
    ],
    'tools'     => [
       'project'      => ['gantt'],
-      'knowbaseitem' => ['tinymce'],
+      'knowbaseitem' => ['tinymce', 'jstree'],
       'reminder'     => ['tinymce']
    ],
+   'management' => [
+      'datacenter'       => [
+         'dcroom' => ['colorpicker', 'gridstack']
+      ]
+   ],
    'config'    => [
       'config'    => ['colorpicker'],
       'commondropdown'  => [
@@ -454,6 +493,7 @@ $CFG_GLPI['javascript'] = [
    ],
    'admin'     => ['colorpicker', 'clipboard'],
    'preference'=> ['colorpicker', 'clipboard'],
+   'self-service' => ['colorpicker', 'tinymce']
 ];
 
 //Maximum time, in miliseconds a saved search should not exeed
